# -*- coding: utf-8 -*-
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.refund_data.electronic_certificate_refund_article import ElectronicCertificateRefundArticle
from yookassa.domain.models.refund_data.refund_data import RequestRefundData
from yookassa.domain.models.refund_data.request.electronic_certificate_refund_data_request import \
    ElectronicCertificateRefundDataRequest


class RefundDataElectronicCertificate(RequestRefundData):
    """
    Данные для возврата платежа через СБП (НСПК).
    """  # noqa: E501

    __electronic_certificate = None
    """Данные от ФЭС НСПК для возврата на электронный сертификат."""

    __articles = None
    """Корзина возврата — список возвращаемых товаров, для оплаты которых использовался электронный сертификат.  Присутствует, если оплата была на [готовой странице ЮKassa](/developers/payment-acceptance/integration-scenarios/manual-integration/other/electronic-certificate/ready-made-payment-form). """  # noqa: E501

    def __init__(self, *args, **kwargs):
        super(RefundDataElectronicCertificate, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.ELECTRONIC_CERTIFICATE:
            self.type = PaymentMethodType.ELECTRONIC_CERTIFICATE

    @property
    def electronic_certificate(self):
        """Возвращает electronic_certificate модели RefundDataElectronicCertificate.

        :return: electronic_certificate модели RefundDataElectronicCertificate.
        :rtype: ElectronicCertificateRefundDataResponse
        """
        return self.__electronic_certificate

    @electronic_certificate.setter
    def electronic_certificate(self, value):
        """Устанавливает electronic_certificate модели RefundDataElectronicCertificate.

        :param value: electronic_certificate модели RefundDataElectronicCertificate.
        :type value: ElectronicCertificateRefundDataResponse
        """
        if isinstance(value, dict):
            self.__electronic_certificate = ElectronicCertificateRefundDataRequest(value)
        elif isinstance(value, ElectronicCertificateRefundDataRequest):
            self.__electronic_certificate = value
        else:
            raise TypeError('Invalid electronic_certificate data type in RefundDataElectronicCertificate.electronic_certificate')

    @property
    def articles(self):
        """Возвращает articles модели RefundDataElectronicCertificate.

        :return: articles модели RefundDataElectronicCertificate.
        :rtype: list[ElectronicCertificateRefundArticle]
        """
        return self.__articles

    @articles.setter
    def articles(self, value):
        """Устанавливает articles модели RefundDataElectronicCertificate.

        :param value: articles модели RefundDataElectronicCertificate.
        :type value: list[ElectronicCertificateRefundArticle]
        """
        if isinstance(value, list):
            articles_array = []
            for articlesData in value:
                if isinstance(articlesData, dict):
                    articles_array.append(ElectronicCertificateRefundArticle(articlesData))
                elif isinstance(articlesData, ElectronicCertificateRefundArticle):
                    articles_array.append(articlesData)
                else:
                    raise TypeError('Invalid articles data type in RefundDataElectronicCertificate.articles')

            self.__articles = articles_array
        else:
            raise TypeError('Invalid articles value type in RefundDataElectronicCertificate')




# -*- coding: utf-8 -*-
from yookassa.domain.common.data_context import DataContext
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payout_data.request.payout_destination_bank_card import \
    PayoutDestinationBankCard as RequestPayoutDestinationBankCard
from yookassa.domain.models.payout_data.request.payout_destination_yoomoney_wallet import \
    PayoutDestinationYooMoneyWallet as RequestPayoutDestinationYooMoneyWallet
from yookassa.domain.models.payout_data.request.payout_destination_sbp import \
    PayoutDestinationSbp as RequestPayoutDestinationSbp
from yookassa.domain.models.payout_data.response.payout_destination_bank_card import \
    PayoutDestinationBankCard as ResponsePayoutDestinationBankCard
from yookassa.domain.models.payout_data.response.payout_destination_yoomoney_wallet import \
    PayoutDestinationYooMoneyWallet as ResponsePayoutDestinationYooMoneyWallet
from yookassa.domain.models.payout_data.response.payout_destination_sbp import \
    PayoutDestinationSbp as ResponsePayoutDestinationSbp
from yookassa.domain.models.payout_data.response.payout_destination_unknown import \
    PayoutDestinationUnknown as ResponsePayoutDestinationUnknown


class PayoutDestinationClassMap(DataContext):
    """
    Сопоставление классов PayoutDestination по типу.
    """  # noqa: E501

    def __init__(self):
        super(PayoutDestinationClassMap, self).__init__(('request', 'response'))

    @property
    def request(self):
        return {
            PaymentMethodType.BANK_CARD: RequestPayoutDestinationBankCard,
            PaymentMethodType.YOO_MONEY: RequestPayoutDestinationYooMoneyWallet,
            PaymentMethodType.SBP: RequestPayoutDestinationSbp,
        }

    @property
    def response(self):
        return {
            PaymentMethodType.BANK_CARD: ResponsePayoutDestinationBankCard,
            PaymentMethodType.YOO_MONEY: ResponsePayoutDestinationYooMoneyWallet,
            PaymentMethodType.SBP: ResponsePayoutDestinationSbp,
            PaymentMethodType.UNKNOWN: ResponsePayoutDestinationUnknown,
        }



# coding: utf-8
from yookassa.domain.common import BaseObject
from yookassa.domain.models import SbpParticipantBank


class SbpBankListResponse(BaseObject):
    """
    Объект ответа, возвращаемого API при запросе списка участников СБП, отсортированный по идентификатору участника в порядке убывания (desc).
    """  # noqa: E501

    __type = None
    """Формат выдачи результатов запроса. Возможное значение: ~`list` (список). """  # noqa: E501

    __items = None
    """Массив платежей."""  # noqa: E501

    @property
    def type(self):
        """
        Возвращает type модели SbpBankListResponse.

        :return: type модели SbpBankListResponse.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели SbpBankListResponse.

        :param value: type модели SbpBankListResponse.
        :type value: str
        """
        self.__type = value

    @property
    def items(self):
        """
        Возвращает items модели SbpBankListResponse.

        :return: items модели SbpBankListResponse.
        :rtype: list[SbpParticipantBank]
        """
        return self.__items

    @items.setter
    def items(self, value):
        """
        Устанавливает items модели SbpBankListResponse.

        :param value: items модели SbpBankListResponse.
        :type value: list[SbpParticipantBank]
        """
        if isinstance(value, list):
            self.__items = [SbpParticipantBank(receipt) for receipt in value]
        else:
            self.__items = value



# -*- coding: utf-8 -*-
from yookassa.domain.models.payment_data.statement.statement import Statement
from yookassa.domain.models.payment_data.statement.statement_type import StatementType
from yookassa.domain.models.payment_data.statement.statement_class_map import StatementClassMap
from yookassa.domain.models.payment_data.statement.statement_factory import StatementFactory



# -*- coding: utf-8 -*-
import json
import sys
import time


class Formatters:
    """
    Класс Formatters.
    """  # noqa: E501

    __tz = time.strftime('%z')

    APP = {
        "name": "appFormatter",
        "config": {
            "format": "[%(asctime)s.%(msecs)03d" + __tz + "] %(levelname)s [ ] [%(name)s] %(message)s %(context)s",
            "datefmt": "%Y-%m-%dT%H:%M:%S",
        }
    }
    JSON = {
        "name": "jsonFormatter",
        "config": {
            "format": json.dumps({
                "message": "%(message)s", "level": "%(levelname)s", "channel": "%(name)s",
                "extra": [], "app_name": "app_main", "app_type": "python", "log_type": "code",
                "es_index_name": "python-main", "timestamp": "%(asctime)s.%(msecs)03d" + __tz,
                "context": "%(context)s"
            }),
            "datefmt": "%Y-%m-%dT%H:%M:%S",
        }
    }
    HEKA = {
        "name": "hekaFormatter",
        "config": {"format": "{}.all.{}.{}.%(message)s:{}".format("dc", "prefix", "type", "postfix")}
    }


class Handlers:
    """
    Класс Handlers.
    """  # noqa: E501

    STREAM = {
        "name": "streamHandler",
        "config": {"class": "logging.StreamHandler", "formatter": Formatters.APP['name'], "stream": sys.stdout},
        "level": 0
    }
    JSON = {
        "name": "jsonHandler",
        "config": {"class": "logging.FileHandler", "formatter": Formatters.JSON['name'], "filename": "json.log"},
        "level": 0
    }
    HEKA = {
        "name": "hekaHandler",
        "config": {"class": "logging.FileHandler", "formatter": Formatters.HEKA['name'], "filename": "heka.log"},
        "level": 0
    }


class Loggers:
    """
    Класс Loggers.
    """  # noqa: E501

    APP = {
        "name": "app",
        "config": {"handlers": [Handlers.STREAM['name'], Handlers.JSON['name']], "level": "DEBUG"}
    }
    HEKA = {
        "name": "heka",
        "config": {"handlers": [Handlers.HEKA['name']], "level": "DEBUG"}  # Handlers.STREAM['name'],
    }



# coding: utf-8
import datetime
import re  # noqa: F401

from yookassa.domain.common import ResponseObject
from yookassa.domain.models.payment_data.response.credit_card import CreditCard
from yookassa.domain.models.payment_method.payment_method_confirmation_factory import PaymentMethodConfirmationFactory
from yookassa.domain.models.payment_method.response.payment_method_holder import PaymentMethodHolder


class PaymentMethodResponse(ResponseObject):
    """Сохраненный способ оплаты."""  # noqa: E501

    __id = None
    """Идентификатор сохраненного способа оплаты."""  # noqa: E501

    __type = None
    """Код способа оплаты. Возможное значение: * ~`bank_card` — банковская карта."""  # noqa: E501

    __card = None
    """Данные банковской карты."""

    __saved = None
    """Признак сохранения способа оплаты для %[автоплатежей](/developers/payment-acceptance/scenario-extensions/recurring-payments/pay-with-saved).  Возможные значения:   * ~`true` — способ оплаты сохранен для автоплатежей и выплат; * ~`false` — способ оплаты не сохранен. """  # noqa: E501

    __status = None
    """Статус проверки и сохранения способа оплаты."""  # noqa: E501

    __holder = None
    """Данные магазина, для которого сохраняется способ оплаты."""  # noqa: E501

    __title = None
    """Название способа оплаты."""  # noqa: E501

    __confirmation = None
    """Выбранный сценарий подтверждения привязки. Присутствует, когда привязка ожидает подтверждения от пользователя."""  # noqa: E501

    @property
    def id(self):
        """Возвращает id модели PaymentMethodResponse.

        :return: id модели PaymentMethodResponse.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """Устанавливает id модели PaymentMethodResponse.

        :param value: id модели PaymentMethodResponse.
        :type value: str
        """
        self.__id = value

    @property
    def type(self):
        """Возвращает type модели PaymentMethodResponse.

        :return: type модели PaymentMethodResponse.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """Устанавливает type модели PaymentMethodResponse.

        :param value: type модели PaymentMethodResponse.
        :type value: str
        """
        self.__type = value

    @property
    def card(self):
        """Возвращает card модели PaymentMethodResponse.

        :return: card модели PaymentMethodResponse.
        :rtype: CreditCard
        """
        return self.__card

    @card.setter
    def card(self, value):
        """Устанавливает card модели PaymentMethodResponse.

        :param value: card модели PaymentMethodResponse.
        :type value: CreditCard
        """
        self.__card = CreditCard(value)

    @property
    def saved(self):
        """Возвращает saved модели PaymentMethodResponse.

        :return: saved модели PaymentMethodResponse.
        :rtype: bool
        """
        return self.__saved

    @saved.setter
    def saved(self, value):
        """Устанавливает saved модели PaymentMethodResponse.

        :param value: saved модели PaymentMethodResponse.
        :type value: bool
        """
        self.__saved = value

    @property
    def status(self):
        """Возвращает status модели PaymentMethodResponse.

        :return: status модели PaymentMethodResponse.
        :rtype: str
        """
        return self.__status

    @status.setter
    def status(self, value):
        """Устанавливает status модели PaymentMethodResponse.

        :param value: status модели PaymentMethodResponse.
        :type value: str
        """
        self.__status = value

    @property
    def holder(self):
        """Возвращает holder модели PaymentMethodResponse.

        :return: holder модели PaymentMethodResponse.
        :rtype: PaymentMethodHolder
        """
        return self.__holder

    @holder.setter
    def holder(self, value):
        """Устанавливает holder модели PaymentMethodResponse.

        :param value: holder модели PaymentMethodResponse.
        :type value: PaymentMethodHolder
        """
        self.__holder = PaymentMethodHolder(value)

    @property
    def title(self):
        """Возвращает title модели PaymentMethodResponse.

        :return: title модели PaymentMethodResponse.
        :rtype: str
        """
        return self.__title

    @title.setter
    def title(self, value):
        """Устанавливает title модели PaymentMethodResponse.

        :param value: title модели PaymentMethodResponse.
        :type value: str
        """
        self.__title = value

    @property
    def confirmation(self):
        """Возвращает confirmation модели PaymentMethodResponse.

        :return: confirmation модели PaymentMethodResponse.
        :rtype: ConfirmationResponse
        """
        return self.__confirmation

    @confirmation.setter
    def confirmation(self, value):
        """Устанавливает confirmation модели PaymentMethodResponse.

        :param value: confirmation модели PaymentMethodResponse.
        :type value: ConfirmationResponse
        """
        self.__confirmation = PaymentMethodConfirmationFactory().create(value, self.context())



# -*- coding: utf-8 -*-
from yookassa.domain.common.request_object import RequestObject

DESCRIPTION_MAX_LENGTH = 128


class DealRequest(RequestObject):
    """
    Объект запроса к API на проведение новой сделки.
    """  # noqa: E501

    __type = None
    """Тип сделки."""

    __fee_moment = None
    """Момент перечисления вам вознаграждения платформы."""

    __description = None
    """Описание сделки."""

    __metadata = None
    """Метаданные привязанные к сделке."""

    @property
    def type(self):
        """
        Возвращает type модели DealRequest.

        :return: type модели DealRequest.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели DealRequest.

        :param value: type модели DealRequest.
        :type value: str
        """
        self.__type = str(value)

    @property
    def fee_moment(self):
        """
        Возвращает fee_moment модели DealRequest.

        :return: fee_moment модели DealRequest.
        :rtype: str
        """
        return self.__fee_moment

    @fee_moment.setter
    def fee_moment(self, value):
        """
        Устанавливает fee_moment модели DealRequest.

        :param value: fee_moment модели DealRequest.
        :type value: str
        """
        self.__fee_moment = str(value)

    @property
    def description(self):
        """
        Возвращает description модели DealRequest.

        :return: description модели DealRequest.
        :rtype: str
        """
        return self.__description

    @description.setter
    def description(self, value):
        """
        Устанавливает description модели DealRequest.

        :param value: description модели DealRequest.
        :type value: str
        """
        cast_value = str(value)
        if cast_value:
            if len(cast_value) <= DESCRIPTION_MAX_LENGTH:
                self.__description = cast_value
            else:
                raise ValueError('The value of the description parameter is too long. Max length is {}'.format(
                    DESCRIPTION_MAX_LENGTH))

    @property
    def metadata(self):
        """
        Возвращает metadata модели DealRequest.

        :return: metadata модели DealRequest.
        :rtype: dict[str, str]
        """
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        """
        Устанавливает metadata модели DealRequest.

        :param value: metadata модели DealRequest.
        :type value: dict[str, str]
        """
        if type(value) is dict:
            self.__metadata = value

    def validate(self):
        """
        Влидация данных модели DealRequest.
        """
        if self.type is None:
            self.__set_validation_error('Deal type not specified')

        if self.fee_moment is None:
            self.__set_validation_error('Deal fee_moment not specified')

    def __set_validation_error(self, message):
        """
        Устанавливает message в Exception при валидации модели DealRequest.

        :param message: message модели Exception.
        :type message: str
        """
        raise ValueError(message)



# coding: utf-8

from yookassa.domain.common import BaseObject
from yookassa.domain.models.invoice import DeliveryMethodType
from yookassa.domain.models.invoice_data.delivery_method import DeliveryMethod


class DeliveryMethodUnknown(DeliveryMethod):
    """Данные для пока неизвестной SDK доставки пользователю ссылки на счет."""  # noqa: E501

    def __init__(self, *args, **kwargs):
        super(DeliveryMethodUnknown, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not DeliveryMethodType.UNKNOWN:
            self.type = DeliveryMethodType.UNKNOWN



# -*- coding: utf-8 -*-
from yookassa.domain.common.type_factory import TypeFactory
from yookassa.domain.models.payment_data.statement import StatementClassMap


class StatementFactory(TypeFactory):
    """
    Фабрика создания объекта Statement по типу.
    """  # noqa: E501

    def __init__(self):
        super(StatementFactory, self).__init__(StatementClassMap())



# -*- coding: utf-8 -*-
from yookassa.domain.common.type_factory import TypeFactory
from yookassa.domain.models.payment_data.statement.delivery_method.delivery_method_class_map import \
    DeliveryMethodClassMap


class DeliveryMethodFactory(TypeFactory):
    """
    Фабрика создания объекта DeliveryMethod по типу.
    """  # noqa: E501

    def __init__(self):
        super(DeliveryMethodFactory, self).__init__(DeliveryMethodClassMap())



# -*- coding: utf-8 -*-

class StatementType:
    """
    Константы, представляющие значения statement_type. Возможные значения:

    * payment_overview - Квитанция по платежу
    """  # noqa: E501

    """
    Список допустимых значений
    """
    PAYMENT_OVERVIEW = 'payment_overview'
    """Квитанция по платежу"""



# coding: utf-8
import re  # noqa: F401

from yookassa.domain.common import BaseObject
from yookassa.domain.models import Amount


class IncomeReceipt(BaseObject):
    """
    Данные чека, зарегистрированного в ФНС. Присутствует, если вы делаете выплату [самозанятому](/developers/payouts/scenario-extensions/self-employed). 
    """  # noqa: E501

    __service_name = None
    """Описание услуги, оказанной получателем выплаты. Не более 50 символов."""  # noqa: E501

    __npd_receipt_id = None
    """Идентификатор чека в сервисе. Пример: ~`208jd98zqe` """  # noqa: E501

    __url = None
    """Ссылка на зарегистрированный чек. Пример: ~`https://www.nalog.gov.ru/api/v1/receipt/<Идентификатор чека>/print` """  # noqa: E501

    __amount = None
    """Сумма, указанная в чеке. Присутствует, если в запросе передавалась сумма для печати в чеке."""  # noqa: E501

    @property
    def service_name(self):
        """
        Возвращает service_name модели IncomeReceipt.

        :return: service_name модели IncomeReceipt.
        :rtype: str
        """
        return self.__service_name

    @service_name.setter
    def service_name(self, value):
        """
        Устанавливает service_name модели IncomeReceipt.

        :param value: service_name модели IncomeReceipt.
        :type value: str
        """
        self.__service_name = value

    @property
    def npd_receipt_id(self):
        """
        Возвращает npd_receipt_id модели IncomeReceipt.

        :return: npd_receipt_id модели IncomeReceipt.
        :rtype: str
        """
        return self.__npd_receipt_id

    @npd_receipt_id.setter
    def npd_receipt_id(self, value):
        """
        Устанавливает npd_receipt_id модели IncomeReceipt.

        :param value: npd_receipt_id модели IncomeReceipt.
        :type value: str
        """
        self.__npd_receipt_id = value

    @property
    def url(self):
        """
        Возвращает url модели IncomeReceipt.

        :return: url модели IncomeReceipt.
        :rtype: str
        """
        return self.__url

    @url.setter
    def url(self, value):
        """
        Устанавливает url модели IncomeReceipt.

        :param value: url модели IncomeReceipt.
        :type value: str
        """
        self.__url = value

    @property
    def amount(self):
        """
        Возвращает amount модели IncomeReceipt.

        :return: amount модели IncomeReceipt.
        :rtype: Amount
        """
        return self.__amount

    @amount.setter
    def amount(self, value):
        """
        Устанавливает amount модели IncomeReceipt.

        :param value: amount модели IncomeReceipt.
        :type value: Amount
        """
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount data type in IncomeReceipt.amount')



# coding: utf-8
import re  # noqa: F401

from yookassa.domain.common import BaseObject


class PaymentPeriod(BaseObject):
    """Период оплаты, за который выставлены начисления и за который вносится оплата."""  # noqa: E501

    __month = None
    """Месяц периода. Например, ~`1` — январь. """  # noqa: E501

    __year = None
    """Год периода. Например, ~`2025`. """  # noqa: E501

    @property
    def month(self):
        """Возвращает month модели PaymentPeriod.

        :return: month модели PaymentPeriod.
        :rtype: int
        """
        return self.__month

    @month.setter
    def month(self, value):
        """Устанавливает month модели PaymentPeriod.

        :param value: month модели PaymentPeriod.
        :type value: int
        """
        if value is not None and value > 12:  # noqa: E501
            raise ValueError("Invalid value for `month`, must be a value less than or equal to `12`")  # noqa: E501
        if value is not None and value < 1:  # noqa: E501
            raise ValueError("Invalid value for `month`, must be a value greater than or equal to `1`")  # noqa: E501
        self.__month = value

    @property
    def year(self):
        """Возвращает year модели PaymentPeriod.

        :return: year модели PaymentPeriod.
        :rtype: int
        """
        return self.__year

    @year.setter
    def year(self, value):
        """Устанавливает year модели PaymentPeriod.

        :param value: year модели PaymentPeriod.
        :type value: int
        """
        self.__year = value





# -*- coding: utf-8 -*-

from yookassa.domain.common import BaseObject


class ReceiptItemSupplier(BaseObject):
    """
    Информация о поставщике товара или услуги (тег в 54 ФЗ — 1224). Можно передавать, если вы отправляете данные для формирования чека по сценарию %[Сначала платеж, потом чек](/developers/payment-acceptance/receipts/54fz/other-services/basics#receipt-after-payment).
    """  # noqa: E501

    __name = None
    """Наименование поставщика (тег в 54 ФЗ — 1225). Параметр предусмотрен форматом фискальных документов (ФФД) и является обязательным, начиная с версии 1.1."""  # noqa: E501

    __inn = None
    """ИНН пользователя (10 или 12 цифр)."""  # noqa: E501

    __phone = None
    """Телефон пользователя. Указывается в формате ITU-T E.164."""  # noqa: E501

    @property
    def name(self):
        """
        Возвращает name модели ReceiptItemSupplier.

        :return: name модели ReceiptItemSupplier.
        :rtype: str
        """
        return self.__name

    @name.setter
    def name(self, value):
        """
        Устанавливает name модели ReceiptItemSupplier.

        :param value: name модели ReceiptItemSupplier.
        :type value: str
        """
        self.__name = str(value)

    @property
    def inn(self):
        """
        Возвращает inn модели ReceiptItemSupplier.

        :return: inn модели ReceiptItemSupplier.
        :rtype: str
        """
        return self.__inn

    @inn.setter
    def inn(self, value):
        """
        Устанавливает inn модели ReceiptItemSupplier.

        :param value: inn модели ReceiptItemSupplier.
        :type value: str
        """
        self.__inn = str(value)

    @property
    def phone(self):
        """
        Возвращает phone модели ReceiptItemSupplier.

        :return: phone модели ReceiptItemSupplier.
        :rtype: str
        """
        return self.__phone

    @phone.setter
    def phone(self, value):
        """
        Устанавливает phone модели ReceiptItemSupplier.

        :param value: phone модели ReceiptItemSupplier.
        :type value: str
        """
        self.__phone = str(value)




# -*- coding: utf-8 -*-
from yookassa.domain.errors.error import ErrorFactory, UnknownError


class ApiError(Exception):
    """
    Неожиданный код ошибки.
    """  # noqa: E501

    __content = None

    __error = None

    HTTP_CODE = 0

    def __init__(self, *args, **kwargs):
        super(ApiError, self).__init__(*args, **kwargs)
        if args[0] is not None:
            error_data = dict(args[0])
            self.__content = error_data

            try:
                factory = ErrorFactory()
                self.__error = factory.create(error_data)
            except (ValueError, TypeError):
                self.__error = UnknownError(error_data)

    @property
    def content(self):
        return self.__content

    @property
    def error(self):
        return self.__error



# -*- coding: utf-8 -*-

class DeliveryMethodType:
    """
    Константы, представляющие значения delivery_method_type. Возможные значения:

    * email - На электронную почту
    """  # noqa: E501

    """
    Список допустимых значений
    """
    EMAIL = 'email'
    """На электронную почту"""



# coding: utf-8
from typing import Dict, Any

from yookassa.domain.common import BaseObject
from yookassa.domain.errors.error_code import ErrorCode


class Error(BaseObject):
    """
    Базовый класс генерируемых объектов Error.
    """  # noqa: E501

    __type = None
    """Тип объекта."""  # noqa: E501

    __id = None
    """Идентификатор ошибки. Используйте его, если возникла необходимость обратиться в техническую поддержку. """  # noqa: E501

    __description = None
    """Подробное описание ошибки."""  # noqa: E501

    __parameter = None
    """Название параметра, из-за которого произошла ошибка."""  # noqa: E501

    __retry_after = None
    """Рекомендуемое количество миллисекунд, спустя которое следует повторить запрос."""  # noqa: E501

    __code = None
    """Код ошибки."""  # noqa: E501

    def __init__(self, data=None):
        super().__init__()
        if data is not None:
            self.from_dict(data)

    @property
    def type(self):
        """Возвращает type модели Error.

        :return: type модели Error.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """Устанавливает type модели Error.

        :param value: type модели Error.
        :type value: str
        """
        self.__type = value

    @property
    def id(self):
        """Возвращает id модели Error.

        :return: id модели Error.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """Устанавливает id модели Error.

        :param value: id модели Error.
        :type value: str
        """
        self.__id = value

    @property
    def description(self):
        """Возвращает description модели Error.

        :return: description модели Error.
        :rtype: str
        """
        return self.__description

    @description.setter
    def description(self, value):
        """Устанавливает description модели Error.

        :param value: description модели Error.
        :type value: str
        """
        self.__description = value

    @property
    def parameter(self):
        """Возвращает parameter модели Error.

        :return: parameter модели Error.
        :rtype: str
        """
        return self.__parameter

    @parameter.setter
    def parameter(self, value):
        """Устанавливает parameter модели Error.

        :param value: parameter модели Error.
        :type value: str
        """
        self.__parameter = value

    @property
    def retry_after(self):
        """Возвращает retry_after модели Error.

        :return: retry_after модели Error.
        :rtype: int
        """
        return self.__retry_after

    @retry_after.setter
    def retry_after(self, value):
        """Устанавливает retry_after модели Error.

        :param value: retry_after модели Error.
        :type value: int
        """
        self.__retry_after = value

    @property
    def code(self):
        """Возвращает code модели Error.

        :return: code модели Error.
        :rtype: str
        """
        return self.__code

    @code.setter
    def code(self, value):
        """Устанавливает code модели Error.

        :param value: code модели Error.
        :type value: str
        """
        self.__code = value

    def from_dict(self, data: Dict[str, Any]):
        """
        Заполняет объект из словаря.

        :param data: Словарь с данными ошибки.
        :type data: Dict
        """
        if 'id' in data:
            self.id = data['id']
        if 'type' in data:
            self.type = data['type']
        if 'code' in data:
            self.code = data['code']
        if 'description' in data:
            self.description = data['description']
        if 'parameter' in data:
            self.parameter = data['parameter']
        if 'retry_after' in data:
            self.retry_after = data['retry_after']


class InvalidRequestError(Error):
    """Ошибка неверного запроса"""

    def __init__(self, *args, **kwargs):
        super(InvalidRequestError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.INVALID_REQUEST


class InvalidCredentialsError(Error):
    """Ошибка неверных учетных данных"""

    def __init__(self, *args, **kwargs):
        super(InvalidCredentialsError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.INVALID_CREDENTIALS


class ForbiddenError(Error):
    """Ошибка доступа запрещен"""

    def __init__(self, *args, **kwargs):
        super(ForbiddenError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.FORBIDDEN


class NotFoundError(Error):
    """Ошибка не найдено"""

    def __init__(self, *args, **kwargs):
        super(NotFoundError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.NOT_FOUND


class GoneError(Error):
    """Ошибка удаленного ресурса"""

    def __init__(self, *args, **kwargs):
        super(GoneError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.GONE


class TooManyRequestsError(Error):
    """Ошибка слишком многих запросов"""

    def __init__(self, *args, **kwargs):
        super(TooManyRequestsError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.TOO_MANY_REQUESTS


class InternalServerError(Error):
    """Ошибка внутреннего сервера"""

    def __init__(self, *args, **kwargs):
        super(InternalServerError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.INTERNAL_SERVER_ERROR


class CommonError(Error):
    """Общая ошибка"""

    def __init__(self, *args, **kwargs):
        super(CommonError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.ERROR


class UnknownError(Error):
    """Неизвестная ошибка"""

    def __init__(self, *args, **kwargs):
        super(UnknownError, self).__init__(*args, **kwargs)
        if self.type is None:
            self.type = ErrorCode.UNKNOWN


class ErrorFactory(object):
    """
    Фабрика создания объекта Error по типу.
    """  # noqa: E501

    _error_class_map = {
        ErrorCode.INVALID_REQUEST: InvalidRequestError,
        ErrorCode.INVALID_CREDENTIALS: InvalidCredentialsError,
        ErrorCode.FORBIDDEN: ForbiddenError,
        ErrorCode.NOT_FOUND: NotFoundError,
        ErrorCode.GONE: GoneError,
        ErrorCode.TOO_MANY_REQUESTS: TooManyRequestsError,
        ErrorCode.INTERNAL_SERVER_ERROR: InternalServerError,
        ErrorCode.ERROR: CommonError,
    }

    def create(self, data: dict):
        """
        Создание экземпляра ошибки из данных

        :param data: словарь с данными code и description
        :return: Экземпляр объекта ошибки по коду
        :raises ValueError: если отсутствует поле 'code'
        :raises TypeError: если data не словарь
        """
        if not isinstance(data, dict):
            raise TypeError('Parameter "data" should be "dict"')

        if 'code' not in data:
            raise ValueError('Parameter "data" should contain "code" field')

        return self._create_from_data(data)

    def _create_from_data(self, data: dict):
        """
        Создание экземпляра ошибки из данных

        :param data: словарь с данными ошибки
        :return: Экземпляр объекта ошибки
        """
        error_class = self._get_error_class(data['code'])
        return error_class(data)

    def _get_error_class(self, code: str):
        """
        Получение класса ошибки на основе кода

        :param code: код ошибки
        :return: класс ошибки
        """
        return self._error_class_map.get(code, UnknownError)





# -*- coding: utf-8 -*-
from yookassa.domain.common import DataContext
from yookassa.domain.models.payment_data.statement.delivery_method.delivery_method_type import DeliveryMethodType
from yookassa.domain.models.payment_data.statement.delivery_method.request.delivery_method_email import \
    DeliveryMethodEmail


class DeliveryMethodClassMap(DataContext):
    """
    Сопоставление классов DeliveryMethod по типу.
    """  # noqa: E501

    def __init__(self):
        super(DeliveryMethodClassMap, self).__init__('request')

    @property
    def request(self):
        return {
            DeliveryMethodType.EMAIL: DeliveryMethodEmail,
        }



# -*- coding: utf-8 -*-
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payout_data.payout_destination import PayoutDestination
from yookassa.domain.models.payout_data.response.credit_card import CreditCard


class PayoutDestinationBankCard(PayoutDestination):
    """
    Данные для выплаты на банковскую карту.
    """  # noqa: E501

    __card = None
    """Данные банковской карты."""  # noqa: E501

    def __init__(self, *args, **kwargs):
        super(PayoutDestinationBankCard, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.BANK_CARD:
            self.type = PaymentMethodType.BANK_CARD

    @property
    def card(self):
        """
        Возвращает card модели PayoutDestinationBankCard.

        :return: card модели PayoutDestinationBankCard.
        :rtype: CreditCard
        """
        return self.__card

    @card.setter
    def card(self, value):
        """
        Устанавливает card модели PayoutDestinationBankCard.

        :param value: card модели PayoutDestinationBankCard.
        :type value: CreditCard
        """
        if isinstance(value, dict):
            self.__card = CreditCard(value)
        elif isinstance(value, CreditCard):
            self.__card = value
        else:
            raise TypeError('Invalid card value type in PayoutDestinationBankCard')




# coding: utf-8
from yookassa.domain.common.response_object import ResponseObject
from yookassa.domain.models import CancellationDetails


class PersonalDataResponse(ResponseObject):
    """
    Объект ответа, возвращаемого API при запросе информации о персональных данных.
    """  # noqa: E501

    __id = None
    """Идентификатор персональных данных, сохраненных в ЮKassa."""  # noqa: E501

    __type = None
    """Тип персональных данных."""  # noqa: E501

    __status = None
    """Статус персональных данных. Возможные значения:  * ~`waiting_for_operation` — данные сохранены, но не использованы при проведении выплаты; * ~`active` — данные сохранены и использованы при проведении выплаты; данные можно использовать повторно до срока, указанного в параметре `expires_at`; * ~`canceled` — хранение данных отменено, данные удалены, инициатор и причина отмены указаны в объекте `cancellation_details` (финальный и неизменяемый статус).  %[Подробнее о жизненном цикле персональных данных](/developers/payouts/scenario-extensions/recipient-check#lifecircle) """  # noqa: E501

    __cancellation_details = None
    """Комментарий к отмене выплаты."""  # noqa: E501

    __created_at = None
    """Время создания персональных данных. Указывается по [UTC](https://ru.wikipedia.org/wiki/Всемирное_координированное_время) и передается в формате [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601). Пример: ~`2017-11-03T11:52:31.827Z` """  # noqa: E501

    __expires_at = None
    """Срок жизни объекта персональных данных — время, до которого вы можете использовать персональные данные при проведении операций. Указывается только для объекта в статусе ~`active`. Указывается по [UTC](https://ru.wikipedia.org/wiki/Всемирное_координированное_время) и передается в формате [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601). Пример: ~`2017-11-03T11:52:31.827Z` """  # noqa: E501

    __metadata = None
    """Любые дополнительные данные, которые нужны вам для работы (например, ваш внутренний идентификатор заказа). Передаются в виде набора пар «ключ-значение» и возвращаются в ответе от ЮKassa. Ограничения: максимум 16 ключей, имя ключа не больше 32 символов, значение ключа не больше 512 символов, тип данных — строка в формате UTF-8. """  # noqa: E501

    @property
    def id(self):
        """
        Возвращает id модели PersonalDataResponse.

        :return: id модели PersonalDataResponse.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """
        Устанавливает id модели PersonalDataResponse.

        :param value: id модели PersonalDataResponse.
        :type value: str
        """
        self.__id = value

    @property
    def type(self):
        """
        Возвращает type модели PersonalDataResponse.

        :return: type модели PersonalDataResponse.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели PersonalDataResponse.

        :param value: type модели PersonalDataResponse.
        :type value: str
        """
        self.__type = value

    @property
    def status(self):
        """
        Возвращает status модели PersonalDataResponse.

        :return: status модели PersonalDataResponse.
        :rtype: str
        """
        return self.__status

    @status.setter
    def status(self, value):
        """
        Устанавливает status модели PersonalDataResponse.

        :param value: status модели PersonalDataResponse.
        :type value: str
        """
        self.__status = value

    @property
    def cancellation_details(self):
        """
        Возвращает cancellation_details модели PersonalDataResponse.

        :return: cancellation_details модели PersonalDataResponse.
        :rtype: CancellationDetails
        """
        return self.__cancellation_details

    @cancellation_details.setter
    def cancellation_details(self, value):
        """
        Устанавливает cancellation_details модели PersonalDataResponse.

        :param value: cancellation_details модели PersonalDataResponse.
        :type value: CancellationDetails
        """
        self.__cancellation_details = CancellationDetails(value)

    @property
    def created_at(self):
        """
        Возвращает created_at модели PersonalDataResponse.

        :return: created_at модели PersonalDataResponse.
        :rtype: datetime
        """
        return self.__created_at

    @created_at.setter
    def created_at(self, value):
        """
        Устанавливает created_at модели PersonalDataResponse.

        :param value: created_at модели PersonalDataResponse.
        :type value: datetime
        """
        self.__created_at = value

    @property
    def expires_at(self):
        """
        Возвращает expires_at модели PersonalDataResponse.

        :return: expires_at модели PersonalDataResponse.
        :rtype: datetime
        """
        return self.__expires_at

    @expires_at.setter
    def expires_at(self, value):
        """
        Устанавливает expires_at модели PersonalDataResponse.

        :param value: expires_at модели PersonalDataResponse.
        :type value: datetime
        """
        self.__expires_at = value

    @property
    def metadata(self):
        """
        Возвращает metadata модели PersonalDataResponse.

        :return: metadata модели PersonalDataResponse.
        :rtype: dict[str, str]
        """
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        """
        Устанавливает metadata модели PersonalDataResponse.

        :param value: metadata модели PersonalDataResponse.
        :type value: dict[str, str]
        """
        self.__metadata = value



# -*- coding: utf-8 -*-
from yookassa.domain.request import PayoutRequest


class PayoutRequestBuilder(object):
    """
    Конструктор запроса на проведение новой выплаты.
    """  # noqa: E501

    def __init__(self):
        self.__request = PayoutRequest()

    def set_amount(self, value):
        """
        Устанавливает amount модели PayoutRequestBuilder.

        :param value: amount модели PayoutRequestBuilder.
        :type value: Amount
        :rtype: PayoutRequestBuilder
        """
        self.__request.amount = value
        return self

    def set_description(self, value):
        """
        Устанавливает description модели PayoutRequestBuilder.

        :param value: description модели PayoutRequestBuilder.
        :type value: str
        :rtype: PayoutRequestBuilder
        """
        self.__request.description = value
        return self

    def set_payout_token(self, value):
        """
        Устанавливает payout_token модели PayoutRequestBuilder.

        :param value: payout_token модели PayoutRequestBuilder.
        :type value: str
        :rtype: PayoutRequestBuilder
        """
        self.__request.payout_token = value
        return self

    def set_payout_destination_data(self, value):
        """
        Устанавливает payout_destination_data модели PayoutRequestBuilder.

        :param value: payout_destination_data модели PayoutRequestBuilder.
        :type value: PayoutDestination
        :rtype: PayoutRequestBuilder
        """
        self.__request.payout_destination_data = value
        return self

    def set_deal(self, value):
        """
        Устанавливает deal модели PayoutRequestBuilder.

        :param value: deal модели PayoutRequestBuilder.
        :type value: PayoutDealInfo
        :rtype: PayoutRequestBuilder
        """
        self.__request.deal = value
        return self

    def set_metadata(self, value):
        """
        Устанавливает metadata модели PayoutRequestBuilder.

        :param value: metadata модели PayoutRequestBuilder.
        :type value: dict[str, str]
        :rtype: PayoutRequestBuilder
        """
        self.__request.metadata = value
        return self

    def build(self):
        """
        Возвращает request модели PayoutRequestBuilder.

        :return: request модели PayoutRequestBuilder.
        :rtype: PayoutRequest
        """
        return self.__request



# -*- coding: utf-8 -*-


class ReceiptType:
    """
    Константы, представляющие значения типа чеков. Возможные значения:

    * payment - Тип чека: приход
    * refund - Тип чека: возврат
    """  # noqa: E501

    PAYMENT = 'payment'
    """Тип чека: приход"""
    REFUND = 'refund'
    """Тип чека: возврат"""


class ReceiptItemAgentType:
    """
    Константы, представляющие значения типа посредника. Возможные значения:

    * banking_payment_agent - Банковский платежный агент
    * banking_payment_subagent - Банковский платежный субагент
    * payment_agent - Платежный агент
    * payment_subagent - Платежный субагент
    * attorney - Поверенный
    * commissioner - Комиссионер
    * agent - Агент
    """  # noqa: E501

    """
    Список допустимых значений
    """
    BANKING_PAYMENT_AGENT = 'banking_payment_agent'
    """Банковский платежный агент"""
    BANKING_PAYMENT_SUBAGENT = 'banking_payment_subagent'
    """Банковский платежный субагент"""
    PAYMENT_AGENT = 'payment_agent'
    """Платежный агент"""
    PAYMENT_SUBAGENT = 'payment_subagent'
    """Платежный субагент"""
    ATTORNEY = 'attorney'
    """Поверенный"""
    COMMISSIONER = 'commissioner'
    """Комиссионер"""
    AGENT = 'agent'
    """Агент"""



# -*- coding: utf-8 -*-
import re

from yookassa.domain.common import BaseObject


class CreditCard(BaseObject):
    """
    Данные банковской карты для выплаты.
    """  # noqa: E501

    __number = None
    """Номер банковской карты. Формат: только цифры, без пробелов. Пример: ~`5555555555554477` """  # noqa: E501

    @property
    def number(self):
        """
        Возвращает number модели CreditCard.

        :return: number модели CreditCard.
        :rtype: str
        """
        return self.__number

    @number.setter
    def number(self, value):
        """
        Устанавливает number модели CreditCard.

        :param value: number модели CreditCard.
        :type value: str
        """
        cast_value = str(value)
        if re.match(r'^[0-9]{16,19}$', cast_value):
            self.__number = cast_value
        else:
            raise ValueError('Invalid card number value')



# coding: utf-8
import re
from typing import Optional, Union, Dict, List

class ProductCode:
    """Класс для формирования тега 1162 на основе кода в формате Data Matrix.""" # noqa: E501

    PREFIX_DATA_MATRIX = '444D'
    """Код типа маркировки DataMatrix""" # noqa: E501

    PREFIX_UNKNOWN = '0000'
    """Код типа маркировки UNKNOWN""" # noqa: E501

    PREFIX_EAN_8 = '4508'
    """Код типа маркировки EAN_8""" # noqa: E501

    PREFIX_EAN_13 = '450D'
    """Код типа маркировки EAN_13""" # noqa: E501

    PREFIX_ITF_14 = '4909'
    """Код типа маркировки ITF_14""" # noqa: E501

    PREFIX_FUR = '5246'
    """Код типа маркировки FUR""" # noqa: E501

    PREFIX_EGAIS_20 = 'C514'
    """Код типа маркировки EGAIS_20""" # noqa: E501

    PREFIX_EGAIS_30 = 'C51E'
    """Код типа маркировки EGAIS_30""" # noqa: E501

    TYPE_UNKNOWN = 'unknown'
    """Тип маркировки UNKNOWN""" # noqa: E501

    TYPE_EAN_8 = 'ean_8'
    """Тип маркировки EAN_8""" # noqa: E501

    TYPE_EAN_13 = 'ean_13'
    """Тип маркировки EAN_13""" # noqa: E501

    TYPE_ITF_14 = 'itf_14'
    """Тип маркировки ITF_14""" # noqa: E501

    TYPE_GS_10 = 'gs_10'
    """Тип маркировки GS_10""" # noqa: E501

    TYPE_GS_1M = 'gs_1m'
    """Тип маркировки GS_1M""" # noqa: E501

    TYPE_SHORT = 'short'
    """Тип маркировки SHORT""" # noqa: E501

    TYPE_FUR = 'fur'
    """Тип маркировки FUR""" # noqa: E501

    TYPE_EGAIS_20 = 'egais_20'
    """Тип маркировки EGAIS_20""" # noqa: E501

    TYPE_EGAIS_30 = 'egais_30'
    """Тип маркировки EGAIS_30""" # noqa: E501

    AI_GTIN = '01'
    """Идентификатор применения (идентификационный номер единицы товара)""" # noqa: E501

    AI_SERIAL = '21'
    """Идентификатор применения (серийный номер)""" # noqa: E501

    AI_SUM = '8005'
    """Дополнительный идентификатор применения (цена единицы измерения товара)""" # noqa: E501

    MAX_PRODUCT_CODE_LENGTH = 30
    """Максимальная длина последовательности для кода продукта unknown""" # noqa: E501

    MAX_MARK_CODE_LENGTH = 32
    """Максимальная длина последовательности для кода маркировки типа unknown""" # noqa: E501

    _prefix = None
    """Код типа маркировки""" # noqa: E501

    _type = None
    """Тип маркировки""" # noqa: E501

    _gtin = None
    """Глобальный номер товарной продукции в единой международной базе товаров GS1 https://ru.wikipedia.org/wiki/GS1. Пример: 04630037591316""" # noqa: E501

    _serial = None
    """Серийный номер товара. Пример: sgEKKPPcS25y5""" # noqa: E501

    _app_identifiers = None
    """Массив дополнительных идентификаторов применения.""" # noqa: E501

    _result = None
    """Сформированный тег 1162. Формат: hex([prefix]+gtin+serial). Пример: 04 36 03 BE F5 14 73  67  45  4b  4b  50  50  63  53  32  35  79  35""" # noqa: E501

    _mark_code_info = None
    """Сформированный код товара (тег в 54 ФЗ — 1163).""" # noqa: E501

    _use_prefix = False
    """Флаг использования кода типа маркировки.""" # noqa: E501

    def __init__(self, code_data_matrix, use_prefix = True):
        """
        Инициализация объекта ProductCode.

        :param code_data_matrix: Строка, расшифрованная из QR-кода
        :param use_prefix: Нужен ли код типа маркировки в результате (True/False или строка с префиксом)
        """
        self._prepare_prefix(use_prefix)

        if code_data_matrix and self._parse_code_matrix_data(code_data_matrix):
            self._result = self.calc_result()

    def __str__(self) -> str:
        """Приведение объекта к строке."""
        return self.get_result()

    @property
    def prefix(self) -> Optional[str]:
        """Возвращает код типа маркировки."""
        return self._prefix

    @prefix.setter
    def prefix(self, value: Union[int, str, None]):
        """Устанавливает код типа маркировки."""
        if value is None or value == '':
            self._prefix = None
            return

        if isinstance(value, int):
            value = hex(value)[2:]
        self._prefix = value.zfill(4).upper()

    @property
    def type(self) -> Optional[str]:
        """Возвращает тип маркировки."""
        return self._type

    @type.setter
    def type(self, value: str):
        """Устанавливает тип маркировки."""
        self._type = value

    @property
    def gtin(self) -> Optional[str]:
        """Возвращает глобальный номер товарной продукции."""
        return self._gtin

    @gtin.setter
    def gtin(self, value: Optional[str]):
        """Устанавливает глобальный номер товарной продукции."""
        self._gtin = value if value not in (None, '') else None

    @property
    def serial(self) -> Optional[str]:
        """Возвращает серийный номер товара."""
        return self._serial

    @serial.setter
    def serial(self, value: Optional[str]):
        """Устанавливает серийный номер товара."""
        self._serial = value if value not in (None, '') else None

    @property
    def app_identifiers(self) -> Optional[List[str]]:
        """Возвращает массив дополнительных идентификаторов применения."""
        return self._app_identifiers

    @app_identifiers.setter
    def app_identifiers(self, value: Optional[List[str]]):
        """Устанавливает массив дополнительных идентификаторов применения."""
        self._app_identifiers = value

    @property
    def use_prefix(self) -> bool:
        """Возвращает флаг использования кода типа маркировки."""
        return self._use_prefix

    @use_prefix.setter
    def use_prefix(self, value: bool):
        """Устанавливает флаг использования кода типа маркировки."""
        self._use_prefix = value

    def get_result(self) -> str:
        """Возвращает сформированный тег 1162."""
        if not self._result:
            self._result = self.calc_result()
        return self._result

    def get_mark_code_info(self) -> Optional[Dict]:
        """Возвращает сформированный код товара (тег в 54 ФЗ — 1163)."""
        return self._mark_code_info

    def set_mark_code_info(self, value: Union[Dict, str]):
        """Устанавливает код товара (тег в 54 ФЗ — 1163)."""
        if isinstance(value, str):
            self._mark_code_info = {self.type: value}
        elif isinstance(value, dict):
            self._mark_code_info = value

    def calc_result(self) -> str:
        """Формирует тег 1162."""
        result = ''

        if not self.validate():
            return result

        if self.use_prefix:
            result = self.prefix if self.prefix else self.PREFIX_DATA_MATRIX

        if self.type == self.TYPE_EAN_8 or self.type == self.TYPE_EAN_13 or self.type == self.TYPE_ITF_14:
            result += self._num_to_hex(self.gtin)
        elif self.type in (self.TYPE_FUR, self.TYPE_EGAIS_20, self.TYPE_EGAIS_30, self.TYPE_UNKNOWN):
            result += self._str_to_hex(self.gtin)
        elif self.type == self.TYPE_SHORT:
            result += self._num_to_hex(self.gtin)
            result += self._str_to_hex(self.serial)
        elif self.type in (self.TYPE_GS_1M, self.TYPE_GS_10):
            result += self._num_to_hex(self.gtin)
            result += self._str_to_hex(self.serial)
            if sum_value := self._get_ai_value(self.AI_SUM):
                result += self._str_to_hex(sum_value)

        return self._chunk_str(result)

    def validate(self) -> bool:
        """Проверяет заполненность необходимых свойств."""
        if not self.type:
            return False

        if self.type in (
                self.TYPE_EAN_8, self.TYPE_EAN_13, self.TYPE_ITF_14,
                self.TYPE_FUR, self.TYPE_EGAIS_20, self.TYPE_EGAIS_30,
                self.TYPE_UNKNOWN
        ):
            return self.gtin is not None
        elif self.type in (self.TYPE_GS_10, self.TYPE_GS_1M, self.TYPE_SHORT):
            return self.gtin is not None and self.serial is not None
        else:
            return False

    def _prepare_prefix(self, use_prefix: Union[bool, str]):
        """Устанавливает prefix и use_prefix в зависимости от входящего параметра."""
        if use_prefix:
            self.use_prefix = True
            if isinstance(use_prefix, (str, int)):
                self.prefix = use_prefix
            else:
                self.prefix = self.PREFIX_UNKNOWN
        else:
            self.use_prefix = False
            self.prefix = None

    def _parse_code_matrix_data(self, code_data_matrix: str) -> bool:
        """Извлекает данные из строки QR-кода и устанавливает свойства."""
        data = self._parse_scan_string(code_data_matrix)
        if not data:
            data = {'type': self.TYPE_UNKNOWN, 'code': code_data_matrix}

        self._fill_data(data)
        return self.validate()

    def _fill_data(self, data: Dict):
        """Заполняет поля объекта из массива данных."""
        self.type = data['type']
        self.prefix = self._get_prefix_by_type(data['type'])

        if self.type in (self.TYPE_EAN_8, self.TYPE_EAN_13, self.TYPE_ITF_14, self.TYPE_FUR, self.TYPE_EGAIS_30):
            self.gtin = data['match1']
            self.set_mark_code_info(self.gtin)
        elif self.type == self.TYPE_EGAIS_20:
            self.gtin = data['match2']
            self.set_mark_code_info(self.gtin)
        elif self.type == self.TYPE_SHORT:
            self.gtin = data['match1']
            self.serial = data['match2']
            self.set_mark_code_info(f"{self.AI_GTIN}{self.gtin}{self.AI_SERIAL}{self.serial}")
        elif self.type in (self.TYPE_GS_1M, self.TYPE_GS_10):
            self.gtin = data['match1']
            if data.get('split') and len(data['split']) > 1:
                self.serial = data['split'].pop(0)
                self.app_identifiers = data['split']
            else:
                self.serial = data['match2']
            self.set_mark_code_info(f"{self.AI_GTIN}{self.gtin}{self.AI_SERIAL}{self.serial}")
        elif self.type == self.TYPE_UNKNOWN:
            self.gtin = data['code'][:self.MAX_PRODUCT_CODE_LENGTH] if len(data['code']) > self.MAX_PRODUCT_CODE_LENGTH else data['code']
            self.serial = data['code'][:self.MAX_MARK_CODE_LENGTH] if len(data['code']) > self.MAX_MARK_CODE_LENGTH else data['code']
            self.set_mark_code_info(self.serial)

    def _parse_scan_string(self, code: str) -> Optional[Dict]:
        """Анализирует строку сканера и возвращает данные."""
        for code_type, rule in self._get_mark_code_rules().items():
            if rule.get('length') and len(code) != rule['length']:
                continue

            match = re.match(rule['pattern'], code)
            if not match:
                continue

            if rule['matches'][0] and not match.group(1):
                continue
            if rule['matches'][1] and not match.group(2):
                continue

            split = None
            if rule.get('split') and match.group(2):
                split = re.split(rule['split'], match.group(2))

            return {
                'type': code_type,
                'code': code,
                'rules': rule['matches'],
                'match1': match.group(1) if rule['matches'][0] else None,
                'match2': match.group(2) if rule['matches'][1] else None,
                'split': split
            }
        return None

    def _get_mark_code_rules(self) -> Dict:
        """Возвращает правила определения типа маркировки."""
        return {
            self.TYPE_GS_1M: {
                'pattern': r'^01(\d{14})21(.+)((91(.+)92(.+))|(93[\w!"%&\'()*+,-./_:;=<>?]{4}(.*)))$',
                'matches': [True, True],
                'split': r'\x1d'
            },
            self.TYPE_GS_10: {
                'pattern': r'^01(\d{14})21(.+)$',
                'matches': [True, True],
                'split': r'\x1d'
            },
            self.TYPE_SHORT: {
                'length': 29,
                'pattern': r'^(\d{14})(.+)$',
                'matches': [True, True]
            },
            self.TYPE_EGAIS_20: {
                'length': 68,
                'pattern': r'^(.{8})(.{33})(.+)$',
                'matches': [False, True]
            },
            self.TYPE_EGAIS_30: {
                'length': 150,
                'pattern': r'^(.{14})(.+)$',
                'matches': [True, False]
            },
            self.TYPE_ITF_14: {
                'length': 14,
                'pattern': r'^(0\d{13})$',
                'matches': [True, False]
            },
            self.TYPE_EAN_13: {
                'length': 13,
                'pattern': r'^(\d{13})$',
                'matches': [True, False]
            },
            self.TYPE_EAN_8: {
                'length': 8,
                'pattern': r'^(\d{8})$',
                'matches': [True, False]
            },
            self.TYPE_FUR: {
                'length': 20,
                'pattern': r'^((\w{2})-(\d{6})-(\w{10}))$',
                'matches': [True, False]
            }
        }

    def _get_ai_value(self, app_identifier: str) -> Optional[str]:
        """Возвращает значение идентификатора применения."""
        if not self.app_identifiers:
            return None

        for item in self.app_identifiers:
            if item.startswith(app_identifier):
                return item[len(app_identifier):]
        return None

    def _get_prefix_by_type(self, code_type: Optional[str] = None) -> str:
        """Возвращает префикс кода товара по типу маркировки."""
        if not code_type:
            code_type = self.type

        prefix_map = {
            self.TYPE_UNKNOWN: self.PREFIX_UNKNOWN,
            self.TYPE_EAN_8: self.PREFIX_EAN_8,
            self.TYPE_EAN_13: self.PREFIX_EAN_13,
            self.TYPE_ITF_14: self.PREFIX_ITF_14,
            self.TYPE_GS_10: self.PREFIX_DATA_MATRIX,
            self.TYPE_GS_1M: self.PREFIX_DATA_MATRIX,
            self.TYPE_SHORT: self.PREFIX_DATA_MATRIX,
            self.TYPE_FUR: self.PREFIX_FUR,
            self.TYPE_EGAIS_20: self.PREFIX_EGAIS_20,
            self.TYPE_EGAIS_30: self.PREFIX_EGAIS_30
        }

        return prefix_map.get(code_type, self.PREFIX_UNKNOWN)

    def _chunk_str(self, string: str) -> str:
        """Разбивает строку на пары символов с пробелами."""
        return ' '.join([string[i:i+2] for i in range(0, len(string), 2)]).upper()

    def _num_to_hex(self, number: str) -> str:
        """Конвертирует число в шестнадцатеричный вид."""
        return self._base_convert(number).zfill(12)

    def _base_convert(self, num_str: str, from_base: int = 10, to_base: int = 16) -> str:
        """Конвертирует число между системами счисления."""
        chars = '0123456789abcdefghijklmnopqrstuvwxyz'
        to_string = chars[:to_base]

        number = [chars.index(c) for c in num_str]
        result = ''

        while number:
            divide = 0
            temp = []

            for digit in number:
                divide = divide * from_base + digit
                if divide >= to_base:
                    temp.append(divide // to_base)
                    divide %= to_base
                elif temp:
                    temp.append(0)

            number = temp
            result = to_string[divide] + result

        return result

    def _str_to_hex(self, string: str) -> str:
        """Конвертирует строку в шестнадцатеричный вид."""
        return ''.join(f"{ord(c):02x}" for c in string)

    def _hex_to_str(self, hex_str: str) -> str:
        """Конвертирует шестнадцатеричную строку в обычную (для тестирования)."""
        return ''.join(chr(int(hex_str[i:i+2], 16)) for i in range(0, len(hex_str), 2))



# -*- coding: utf-8 -*-
from yookassa.domain.common.confirmation_type import ConfirmationType
from yookassa.domain.models.confirmation.confirmation import Confirmation


class ConfirmationMobileApplication(Confirmation):
    """
    Сценарий, при котором необходимо совершить действия в мобильном приложении (например, в приложении интернет-банка).
    """  # noqa: E501

    __confirmation_url = None
    """Диплинк на мобильное приложение, в котором пользователь подтверждает платеж."""  # noqa: E501

    def __init__(self, *args, **kwargs):
        super(ConfirmationMobileApplication, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not ConfirmationType.MOBILE_APPLICATION:
            self.type = ConfirmationType.MOBILE_APPLICATION

    @property
    def confirmation_url(self):
        """
        Возвращает confirmation_url модели ConfirmationMobileApplication.

        :return: confirmation_url модели ConfirmationMobileApplication.
        :rtype: str
        """
        return self.__confirmation_url

    @confirmation_url.setter
    def confirmation_url(self, value):
        """
        Устанавливает confirmation_url модели ConfirmationMobileApplication.

        :param value: confirmation_url модели ConfirmationMobileApplication.
        :type value: str
        """
        self.__confirmation_url = value



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.notification.webhook_notification_types import WebhookNotificationType, \
    WebhookNotificationEventType
from yookassa.domain.response import DealResponse
from yookassa.domain.response.payment_response import PaymentResponse
from yookassa.domain.response.payout_response import PayoutResponse
from yookassa.domain.response.refund_response import RefundResponse


class WebhookNotification(BaseObject):
    """
    Базовый класс генерируемых объектов WebhookNotification.
    """  # noqa: E501

    __type = None
    """Тип уведомления."""  # noqa: E501

    __event = None
    """Событие, о котором уведомляет ЮKassa."""  # noqa: E501

    __object = None
    """Объект, с которым произошло событие: payment — платеж, refund — возврат, payout — выплата, deal — сделка."""  # noqa: E501

    @property
    def type(self):
        """
        Возвращает type модели WebhookNotification.

        :return: type модели WebhookNotification.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели WebhookNotification.

        :param value: type модели WebhookNotification.
        :type value: str
        """
        self.__type = value

    @property
    def event(self):
        """
        Возвращает event модели WebhookNotification.

        :return: event модели WebhookNotification.
        :rtype: str
        """
        return self.__event

    @event.setter
    def event(self, value):
        """
        Устанавливает event модели WebhookNotification.

        :param value: event модели WebhookNotification.
        :type value: str
        """
        self.__event = value

    @property
    def object(self):
        """
        Возвращает object модели WebhookNotification.

        :return: object модели WebhookNotification.
        :rtype: PaymentResponse
        """
        return self.__object

    @object.setter
    def object(self, value):
        """
        Устанавливает object модели WebhookNotification.

        :param value: object модели WebhookNotification.
        :type value: PaymentResponse
        """
        if isinstance(value, dict) and value:
            self.__object = PaymentResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class PaymentWebhookNotification(WebhookNotification):
    """
    Объект с информацией о платеже, уведомление о котором хранится в текущем объекте.
    """  # noqa: E501
    pass


class RefundWebhookNotification(WebhookNotification):
    """
    Объект с информацией о возврате, уведомление о котором хранится в текущем объекте.
    """  # noqa: E501

    @property
    def object(self):
        """
        Возвращает object модели RefundWebhookNotification.

        :return: object модели RefundWebhookNotification.
        :rtype: RefundResponse
        """
        return self.__object

    @object.setter
    def object(self, value):
        """
        Устанавливает object модели RefundWebhookNotification.

        :param value: object модели RefundWebhookNotification.
        :type value: RefundResponse
        """
        if isinstance(value, dict) and value:
            self.__object = RefundResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class DealWebhookNotification(WebhookNotification):
    """
    Объект с информацией о сделке, уведомление о котором хранится в текущем объекте.
    """  # noqa: E501

    @property
    def object(self):
        """
        Возвращает object модели DealWebhookNotification.

        :return: object модели DealWebhookNotification.
        :rtype: DealResponse
        """
        return self.__object

    @object.setter
    def object(self, value):
        """
        Устанавливает object модели DealWebhookNotification.

        :param value: object модели DealWebhookNotification.
        :type value: DealResponse
        """
        if isinstance(value, dict) and value:
            self.__object = DealResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class PayoutWebhookNotification(WebhookNotification):
    """
    Объект с информацией о выплате, уведомление о котором хранится в текущем объекте.
    """  # noqa: E501

    @property
    def object(self):
        """
        Возвращает object модели PayoutWebhookNotification.

        :return: object модели PayoutWebhookNotification.
        :rtype: PayoutResponse
        """
        return self.__object

    @object.setter
    def object(self, value):
        """
        Устанавливает object модели PayoutWebhookNotification.

        :param value: object модели PayoutWebhookNotification.
        :type value: PayoutResponse
        """
        if isinstance(value, dict) and value:
            self.__object = PayoutResponse(value)
        elif not value:
            raise ValueError('Parameter object is empty')
        else:
            raise TypeError('Invalid object type')


class WebhookNotificationFactory(object):
    """
    Фабрика создания объекта WebhookNotification по типу.
    """  # noqa: E501

    def create(self, data):
        """
        Создание экземпляра из данных

        :param data: словарь с данными type и event
        :return: Экземпляр объекта по типу
        """
        if isinstance(data, dict):
            if 'type' in data and 'event' in data:
                return self.__get_instance(data)
            else:
                raise ValueError('Parameter "data" should contain "type" and "event" fields')
        else:
            raise TypeError('Parameter "data" should be "dict"')

    def __get_instance(self, data):
        """
        Получение экземпляра из данных

        :param data: словарь с данными type и event
        :return: Экземпляр объекта по типу
        """
        class_object = self.__get_class_object(data)
        return class_object(data)

    @staticmethod
    def __get_class_object(data):
        """
        Получение экземпляра из данных

        :param data: словарь с данными type и event
        :return: Экземпляр объекта по типу
        """
        if data['type'] == WebhookNotificationType.NOTIFICATION:
            if data['event'] == WebhookNotificationEventType.REFUND_SUCCEEDED:
                return RefundWebhookNotification
            elif data['event'] == WebhookNotificationEventType.DEAL_CLOSED:
                return DealWebhookNotification
            elif data['event'] == WebhookNotificationEventType.PAYOUT_SUCCEEDED or \
                    data['event'] == WebhookNotificationEventType.PAYOUT_CANCELED:
                return PayoutWebhookNotification
            else:
                return WebhookNotification
        else:
            raise TypeError('Parameter "data" should contain "type" field')



# -*- coding: utf-8 -*-
from yookassa.domain.exceptions.api_error import ApiError


class ResponseProcessingError(ApiError):
    """
    Запрос был принят на обработку, но она не завершена.
    """  # noqa: E501
    HTTP_CODE = 202



# -*- coding: utf-8 -*-
from yookassa.domain.common.data_context import DataContext
from yookassa.domain.models.self_employed_data.confirmation import SelfEmployedConfirmationType
from yookassa.domain.models.self_employed_data.request.confirmation_redirect import \
    SelfEmployedConfirmationRedirect as RequestConfirmationRedirect
from yookassa.domain.models.self_employed_data.response.confirmation_redirect import \
    SelfEmployedConfirmationRedirect as ResponseConfirmationRedirect


class SelfEmployedConfirmationClassMap(DataContext):
    """
    Сопоставление классов SelfEmployedConfirmation по типу.
    """  # noqa: E501

    def __init__(self):
        super(SelfEmployedConfirmationClassMap, self).__init__(('request', 'response'))

    @property
    def request(self):
        return {
            SelfEmployedConfirmationType.REDIRECT: RequestConfirmationRedirect,
        }

    @property
    def response(self):
        return {
            SelfEmployedConfirmationType.REDIRECT: ResponseConfirmationRedirect,
        }



# coding: utf-8
import re  # noqa: F401

from yookassa.domain.common import BaseObject


class PaymentOrderRecipientBankUtilities(BaseObject):

    __name = None
    """Название банка получателя."""  # noqa: E501

    __bic = None
    """БИК банка получателя."""  # noqa: E501

    __account = None
    """Счет получателя в банке."""  # noqa: E501

    __correspondent_account = None
    """Корреспондентский счет банка получателя."""  # noqa: E501

    @property
    def name(self):
        """Возвращает name модели PaymentOrderRecipientBank.

        :return: name модели PaymentOrderRecipientBank.
        :rtype: str
        """
        return self.__name

    @name.setter
    def name(self, value):
        """Устанавливает name модели PaymentOrderRecipientBank.

        :param value: name модели PaymentOrderRecipientBank.
        :type value: str
        """
        self.__name = value

    @property
    def bic(self):
        """Возвращает bic модели PaymentOrderRecipientBank.

        :return: bic модели PaymentOrderRecipientBank.
        :rtype: str
        """
        return self.__bic

    @bic.setter
    def bic(self, value):
        """Устанавливает bic модели PaymentOrderRecipientBank.

        :param value: bic модели PaymentOrderRecipientBank.
        :type value: str
        """
        if value is not None and not re.search(r'^[0-9]{9}$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `bic`, must be a follow pattern or equal to `/[0-9]{9}/`")  # noqa: E501
        self.__bic = value

    @property
    def account(self):
        """Возвращает account модели PaymentOrderRecipientBank.

        :return: account модели PaymentOrderRecipientBank.
        :rtype: str
        """
        return self.__account

    @account.setter
    def account(self, value):
        """Устанавливает account модели PaymentOrderRecipientBank.

        :param value: account модели PaymentOrderRecipientBank.
        :type value: str
        """
        self.__account = value

    @property
    def correspondent_account(self):
        """Возвращает correspondent_account модели PaymentOrderRecipientBank.

        :return: correspondent_account модели PaymentOrderRecipientBank.
        :rtype: str
        """
        return self.__correspondent_account

    @correspondent_account.setter
    def correspondent_account(self, value):
        """Устанавливает correspondent_account модели PaymentOrderRecipientBank.

        :param value: correspondent_account модели PaymentOrderRecipientBank.
        :type value: str
        """
        self.__correspondent_account = value





# -*- coding: utf-8 -*-
import uuid

from yookassa.client import ApiClient
from yookassa.domain.common import HttpVerb
from yookassa.domain.request import SelfEmployedRequest
from yookassa.domain.response import SelfEmployedResponse


class SelfEmployed:
    """
    Класс, представляющий модель SelfEmployed.
    """  # noqa: E501

    base_path = '/self_employed'

    def __init__(self):
        self.client = ApiClient()

    @classmethod
    def find_one(cls, self_employed_id):
        """
        Возвращает информацию о сомозанятом

        :param self_employed_id: Уникальный идентификатор сомозанятого
        :return: SelfEmployedResponse Объект ответа, возвращаемого API при запросе информации о сомозанятом
        """
        instance = cls()
        if not isinstance(self_employed_id, str) or not self_employed_id:
            raise ValueError('Invalid self_employed_id value')

        path = instance.base_path + '/' + self_employed_id
        response = instance.client.request(HttpVerb.GET, path)
        return SelfEmployedResponse(response)

    @classmethod
    def create(cls, params, idempotency_key=None):
        """
        Создание сомозанятого

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: SelfEmployedResponse Объект ответа, возвращаемого API при запросе информации о сомозанятом
        """
        instance = cls()
        path = cls.base_path

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = SelfEmployedRequest(params)
        elif isinstance(params, SelfEmployedRequest):
            params_object = params
        else:
            raise TypeError('Invalid params value type')

        response = instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return SelfEmployedResponse(response)



# coding: utf-8

from yookassa.domain.common import BaseObject
from yookassa.domain.models import Amount
from yookassa.domain.models.settings import FiscalizationData


class Me(BaseObject):
    """
    Объект настроек магазина или шлюза (Me) — актуальная информация о настройках магазина или шлюза.
    """  # noqa: E501

    __account_id = None
    """Идентификатор магазина или шлюза."""

    __status = None
    """Статус магазина или шлюза. Возможные значения:  * ~`enabled` — подключен к ЮKassa, может проводить платежи или выплаты; * ~`disabled` — не может проводить платежи или выплаты (еще не подключен, закрыт или временно не работает). """  # noqa: E501

    __test = None
    """Это тестовый магазин или шлюз. """  # noqa: E501

    __fiscalization = None
    """Настройки магазина для отправки чеков в налоговую."""

    __fiscalization_enabled = None
    """Устаревший параметр, который раньше использовался для определения настроек отправки чеков в налоговую. Сохранен для поддержки обратной совместимости, в новых версиях API может быть удален.  Используйте объект ~`fiscalization`, чтобы определить, какие у магазина настройки отправки чеков. """  # noqa: E501

    __payment_methods = None
    """Список %[способов оплаты](/developers/payment-acceptance/getting-started/payment-methods#all), доступных магазину. Присутствует, если вы запрашивали настройки магазина. """  # noqa: E501

    __itn = None
    """ИНН магазина (10 или 12 цифр). Присутствует, если вы запрашивали настройки магазина."""

    __payout_methods = None
    """Список способов получения выплат, доступных шлюзу. Возможные значения:  * ~`bank_card` — выплаты на банковские карты; * ~`yoo_money` — выплаты на кошельки ЮMoney; * ~`sbp` — выплаты через СБП.  Присутствует, если вы запрашивали настройки шлюза. """  # noqa: E501

    __name = None
    """Название шлюза, которое отображается в личном кабинете ЮKassa. Присутствует, если вы запрашивали настройки шлюза."""  # noqa: E501

    __payout_balance = None
    """Баланс вашего шлюза. Присутствует, если вы запрашивали настройки шлюза."""

    @property
    def account_id(self):
        """
        Возвращает account_id модели Me.

        :return: account_id модели Me.
        :rtype: str
        """
        return self.__account_id

    @account_id.setter
    def account_id(self, value):
        """
        Устанавливает account_id модели Me.

        :param value: account_id модели Me.
        :type value: str
        """
        self.__account_id = value

    @property
    def status(self):
        """
        Возвращает status модели Me.

        :return: status модели Me.
        :rtype: str
        """
        return self.__status

    @status.setter
    def status(self, value):
        """
        Устанавливает status модели Me.

        :param value: status модели Me.
        :type value: str
        """
        self.__status = value

    @property
    def test(self):
        """
        Возвращает test модели Me.

        :return: test модели Me.
        :rtype: bool
        """
        return self.__test

    @test.setter
    def test(self, value):
        """
        Устанавливает test модели Me.

        :param value: test модели Me.
        :type value: bool
        """
        self.__test = value

    @property
    def fiscalization(self):
        """
        Возвращает fiscalization модели Me.

        :return: fiscalization модели Me.
        :rtype: FiscalizationData
        """
        return self.__fiscalization

    @fiscalization.setter
    def fiscalization(self, value):
        """
        Устанавливает fiscalization модели Me.

        :param value: fiscalization модели Me.
        :type value: FiscalizationData
        """
        if isinstance(value, dict):
            self.__fiscalization = FiscalizationData(value)

    @property
    def fiscalization_enabled(self):
        """
        Возвращает fiscalization_enabled модели Me.

        :return: fiscalization_enabled модели Me.
        :rtype: bool
        """
        return self.__fiscalization_enabled

    @fiscalization_enabled.setter
    def fiscalization_enabled(self, value):
        """
        Устанавливает fiscalization_enabled модели Me.

        :param value: fiscalization_enabled модели Me.
        :type value: bool
        """
        self.__fiscalization_enabled = value

    @property
    def payment_methods(self):
        """
        Возвращает payment_methods модели Me.

        :return: payment_methods модели Me.
        :rtype: list[str]
        """
        return self.__payment_methods

    @payment_methods.setter
    def payment_methods(self, value):
        """
        Устанавливает payment_methods модели Me.

        :param value: payment_methods модели Me.
        :type value: list[str]
        """
        self.__payment_methods = value

    @property
    def itn(self):
        """
        Возвращает itn модели Me.

        :return: itn модели Me.
        :rtype: str
        """
        return self.__itn

    @itn.setter
    def itn(self, value):
        """
        Устанавливает itn модели Me.

        :param value: itn модели Me.
        :type value: str
        """
        self.__itn = value

    @property
    def payout_methods(self):
        """
        Возвращает payout_methods модели Me.

        :return: payout_methods модели Me.
        :rtype: list[str]
        """
        return self.__payout_methods

    @payout_methods.setter
    def payout_methods(self, value):
        """
        Устанавливает payout_methods модели Me.

        :param value: payout_methods модели Me.
        :type value: list[str]
        """
        self.__payout_methods = value

    @property
    def name(self):
        """
        Возвращает name модели Me.

        :return: name модели Me.
        :rtype: str
        """
        return self.__name

    @name.setter
    def name(self, value):
        """
        Устанавливает name модели Me.

        :param value: name модели Me.
        :type value: str
        """
        self.__name = value

    @property
    def payout_balance(self):
        """
        Возвращает payout_balance модели Me.

        :return: payout_balance модели Me.
        :rtype: Amount
        """
        return self.__payout_balance

    @payout_balance.setter
    def payout_balance(self, value):
        """
        Устанавливает payout_balance модели Me.

        :param value: payout_balance модели Me.
        :type value: Amount
        """
        if isinstance(value, dict):
            self.__payout_balance = Amount(value)



# coding: utf-8
import datetime
import re  # noqa: F401

from yookassa.domain.common import TypeFactory, DataContext
from yookassa.domain.common.request_object import RequestObject
from yookassa.domain.models.personal_data import PersonalDataType


class PayoutStatementRecipientPersonalDataRequest(RequestObject):
    """
    Объект запроса к API на создание персональных данных.
    """  # noqa: E501

    __type = None
    """Значение — payout_statement_recipient. Тип персональных данных."""  # noqa: E501

    __last_name = None
    """Фамилия пользователя."""  # noqa: E501

    __first_name = None
    """Имя пользователя."""  # noqa: E501

    __middle_name = None
    """Отчество пользователя. Обязательный параметр, если есть в паспорте."""  # noqa: E501

    __metadata = None
    """Любые дополнительные данные, которые нужны вам для работы (например, ваш внутренний идентификатор заказа). Передаются в виде набора пар «ключ-значение» и возвращаются в ответе от ЮKassa. Ограничения: максимум 16 ключей, имя ключа не больше 32 символов, значение ключа не больше 512 символов, тип данных — строка в формате UTF-8. """  # noqa: E501

    __birthdate = None
    """Дата рождения. Передается в формате ISO 8601"""  # noqa: E501

    @property
    def type(self):
        """
        Возвращает type модели PayoutStatementRecipientPersonalDataRequest.

        :return: type модели PayoutStatementRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели PayoutStatementRecipientPersonalDataRequest.

        :param value: type модели PayoutStatementRecipientPersonalDataRequest.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        self.__type = str(value)

    @property
    def last_name(self):
        """
        Возвращает last_name модели PayoutStatementRecipientPersonalDataRequest.

        :return: last_name модели PayoutStatementRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__last_name

    @last_name.setter
    def last_name(self, value):
        """
        Устанавливает last_name модели PayoutStatementRecipientPersonalDataRequest.

        :param value: last_name модели PayoutStatementRecipientPersonalDataRequest.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `last_name`, must not be `None`")  # noqa: E501
        if value is not None and len(value) > 200:
            raise ValueError("Invalid value for `last_name`, length must be less than or equal to `200`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `last_name`, length must be greater than or equal to `1`")  # noqa: E501
        if value is not None and not re.search(r'^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `last_name`, must be a follow pattern or equal to `/^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$/`")  # noqa: E501
        self.__last_name = value

    @property
    def first_name(self):
        """
        Возвращает first_name модели PayoutStatementRecipientPersonalDataRequest.

        :return: first_name модели PayoutStatementRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__first_name

    @first_name.setter
    def first_name(self, value):
        """
        Устанавливает first_name модели PayoutStatementRecipientPersonalDataRequest.

        :param value: first_name модели PayoutStatementRecipientPersonalDataRequest.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `first_name`, must not be `None`")  # noqa: E501
        if value is not None and len(value) > 100:
            raise ValueError("Invalid value for `first_name`, length must be less than or equal to `100`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `first_name`, length must be greater than or equal to `1`")  # noqa: E501
        if value is not None and not re.search(r'^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `first_name`, must be a follow pattern or equal to `/^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$/`")  # noqa: E501
        self.__first_name = value

    @property
    def middle_name(self):
        """
        Возвращает middle_name модели PayoutStatementRecipientPersonalDataRequest.

        :return: middle_name модели PayoutStatementRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__middle_name

    @middle_name.setter
    def middle_name(self, value):
        """
        Устанавливает middle_name модели PayoutStatementRecipientPersonalDataRequest.

        :param value: middle_name модели PayoutStatementRecipientPersonalDataRequest.
        :type value: str
        """
        if value is not None and len(value) > 200:
            raise ValueError("Invalid value for `middle_name`, length must be less than or equal to `200`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `middle_name`, length must be greater than or equal to `1`")  # noqa: E501
        if value is not None and not re.search(r'^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `middle_name`, must be a follow pattern or equal to `/^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$/`")  # noqa: E501
        self.__middle_name = value

    @property
    def metadata(self):
        """
        Возвращает metadata модели PayoutStatementRecipientPersonalDataRequest.

        :return: metadata модели PayoutStatementRecipientPersonalDataRequest.
        :rtype: dict[str, str]
        """
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        """
        Устанавливает metadata модели PayoutStatementRecipientPersonalDataRequest.

        :param value: metadata модели PayoutStatementRecipientPersonalDataRequest.
        :type value: dict[str, str]
        """
        self.__metadata = value

    @property
    def birthdate(self):
        """
        Возвращает birthdate модели PayoutStatementRecipientPersonalDataRequest.

        :return: birthdate модели PayoutStatementRecipientPersonalDataRequest.
        :rtype: datetime
        """
        return self.__birthdate

    @birthdate.setter
    def birthdate(self, value):
        """
        Устанавливает metadata модели PayoutStatementRecipientPersonalDataRequest.

        :param value: metadata модели PayoutStatementRecipientPersonalDataRequest.
        :type value: str
        """
        try:
            datetime.datetime.strptime(value, '%Y-%m-%d')
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY-MM-DD")

        self.__birthdate = value


    def validate(self):
        """
        Влидация данных модели PayoutStatementRecipientPersonalDataRequest.
        """
        if self.type is None or self.last_name is None or self.first_name is None or self.birthdate is None:
            self.__set_validation_error('Type, last_name, first_name and birthdate values cannot be empty in PayoutStatementRecipientPersonalDataRequest')  # noqa: E501

    def __set_validation_error(self, message):
        """
        Устанавливает message в Exception при валидации модели PayoutStatementRecipientPersonalDataRequest.

        :param message: message модели Exception.
        :type message: str
        """
        raise ValueError(message)

class SbpPayoutRecipientPersonalDataRequest(RequestObject):

    __type = None
    """Тип персональных данных."""  # noqa: E501

    __last_name = None
    """Фамилия пользователя."""  # noqa: E501

    __first_name = None
    """Имя пользователя."""  # noqa: E501

    __middle_name = None
    """Отчество пользователя. Обязательный параметр, если есть в паспорте."""  # noqa: E501

    __metadata = None
    """Любые дополнительные данные, которые нужны вам для работы (например, ваш внутренний идентификатор заказа). Передаются в виде набора пар «ключ-значение» и возвращаются в ответе от ЮKassa. Ограничения: максимум 16 ключей, имя ключа не больше 32 символов, значение ключа не больше 512 символов, тип данных — строка в формате UTF-8. """  # noqa: E501

    @property
    def type(self):
        """
        Возвращает type модели SbpPayoutRecipientPersonalDataRequest.

        :return: type модели SbpPayoutRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели SbpPayoutRecipientPersonalDataRequest.

        :param value: type модели SbpPayoutRecipientPersonalDataRequest.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        self.__type = str(value)

    @property
    def last_name(self):
        """
        Возвращает last_name модели SbpPayoutRecipientPersonalDataRequest.

        :return: last_name модели SbpPayoutRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__last_name

    @last_name.setter
    def last_name(self, value):
        """
        Устанавливает last_name модели SbpPayoutRecipientPersonalDataRequest.

        :param value: last_name модели SbpPayoutRecipientPersonalDataRequest.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `last_name`, must not be `None`")  # noqa: E501
        if value is not None and len(value) > 200:
            raise ValueError("Invalid value for `last_name`, length must be less than or equal to `200`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `last_name`, length must be greater than or equal to `1`")  # noqa: E501
        if value is not None and not re.search(r'^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `last_name`, must be a follow pattern or equal to `/^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$/`")  # noqa: E501
        self.__last_name = value

    @property
    def first_name(self):
        """
        Возвращает first_name модели SbpPayoutRecipientPersonalDataRequest.

        :return: first_name модели SbpPayoutRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__first_name

    @first_name.setter
    def first_name(self, value):
        """
        Устанавливает first_name модели SbpPayoutRecipientPersonalDataRequest.

        :param value: first_name модели SbpPayoutRecipientPersonalDataRequest.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `first_name`, must not be `None`")  # noqa: E501
        if value is not None and len(value) > 100:
            raise ValueError("Invalid value for `first_name`, length must be less than or equal to `100`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `first_name`, length must be greater than or equal to `1`")  # noqa: E501
        if value is not None and not re.search(r'^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `first_name`, must be a follow pattern or equal to `/^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$/`")  # noqa: E501
        self.__first_name = value

    @property
    def middle_name(self):
        """
        Возвращает middle_name модели SbpPayoutRecipientPersonalDataRequest.

        :return: middle_name модели SbpPayoutRecipientPersonalDataRequest.
        :rtype: str
        """
        return self.__middle_name

    @middle_name.setter
    def middle_name(self, value):
        """
        Устанавливает middle_name модели SbpPayoutRecipientPersonalDataRequest.

        :param value: middle_name модели SbpPayoutRecipientPersonalDataRequest.
        :type value: str
        """
        if value is not None and len(value) > 200:
            raise ValueError("Invalid value for `middle_name`, length must be less than or equal to `200`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `middle_name`, length must be greater than or equal to `1`")  # noqa: E501
        if value is not None and not re.search(r'^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$', value):  # noqa: E501
            raise ValueError(r"Invalid value for `middle_name`, must be a follow pattern or equal to `/^[—–‐\-a-zA-Zа-яёА-ЯЁ ]*$/`")  # noqa: E501
        self.__middle_name = value

    @property
    def metadata(self):
        """
        Возвращает metadata модели SbpPayoutRecipientPersonalDataRequest.

        :return: metadata модели SbpPayoutRecipientPersonalDataRequest.
        :rtype: dict[str, str]
        """
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        """
        Устанавливает metadata модели SbpPayoutRecipientPersonalDataRequest.

        :param value: metadata модели SbpPayoutRecipientPersonalDataRequest.
        :type value: dict[str, str]
        """
        self.__metadata = value

    def validate(self):
        """
        Влидация данных модели SbpPayoutRecipientPersonalDataRequest.
        """
        if self.type is None or self.last_name is None or self.first_name is None:
            self.__set_validation_error('Type, last_name and first_name values cannot be empty in SbpPayoutRecipientPersonalDataRequest')  # noqa: E501

    def __set_validation_error(self, message):
        """
        Устанавливает message в Exception при валидации модели SbpPayoutRecipientPersonalDataRequest.

        :param message: message модели Exception.
        :type message: str
        """
        raise ValueError(message)


class PersonalDataRequestClassMap(DataContext):
    """
    Сопоставление классов PersonalDataRequest по типу.
    """  # noqa: E501

    def __init__(self):
        super(PersonalDataRequestClassMap, self).__init__(('request'))

    @property
    def request(self):
        return {
            PersonalDataType.PAYOUT_STATEMENT_RECIPIENT: PayoutStatementRecipientPersonalDataRequest,
            PersonalDataType.SBP_PAYOUT_RECIPIENT: SbpPayoutRecipientPersonalDataRequest,
        }


class PersonalDataRequestFactory(TypeFactory):
    """
    Фабрика создания объекта PersonalDataRequest по типу.
    """  # noqa: E501

    def __init__(self):
        super(PersonalDataRequestFactory, self).__init__(PersonalDataRequestClassMap())



# -*- coding: utf-8 -*-
from yookassa.domain.exceptions.api_error import ApiError


class InternalServerError(ApiError):
    """
    Технические неполадки на стороне ЮKassa. Результат обработки запроса неизвестен. Повторите запрос позднее с тем же ключом идемпотентности.
    Рекомендуется повторять запрос с периодичностью один раз в минуту до тех пор, пока ЮKassa не сообщит результат обработки операции.
    """  # noqa: E501
    HTTP_CODE = 500



# -*- coding: utf-8 -*-
import re

from yookassa.domain.models.payment_data.statement.delivery_method.delivery_method import DeliveryMethod
from yookassa.domain.models.payment_data.statement.delivery_method.delivery_method_type import DeliveryMethodType


class DeliveryMethodEmail(DeliveryMethod):
    """
    Способ доставки справки.
    """  # noqa: E501

    __email = None
    """Электронная почта пользователя для отправки справки. """ # noqa: E501

    def __init__(self, *args, **kwargs):
        super(DeliveryMethodEmail, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not DeliveryMethodType.EMAIL:
            self.type = DeliveryMethodType.EMAIL

    @property
    def email(self):
        """Возвращает email модели DeliveryMethodEmail.

        :return: email модели DeliveryMethodEmail.
        :rtype: str
        """
        return self.__email

    @email.setter
    def email(self, value):
        """Устанавливает email модели DeliveryMethodEmail.

        :param value: email модели DeliveryMethodEmail.
        :type value: str
        """
        cast_value = str(value)
        if cast_value is None:  # noqa: E501
            raise ValueError("Invalid value for `email`, must not be `None`")  # noqa: E501
        elif re.match(r"^[^@]+@[^@]+\.[^@]+$", cast_value):
            self.__email = cast_value
        else:
            raise ValueError('Invalid email value type')



# coding: utf-8
import datetime
import re  # noqa: F401

from yookassa.domain.common import BaseObject


class PaymentOrder(BaseObject):
    """Платежное поручение — распоряжение на перевод банку для оплаты жилищно-коммунальных услуг (ЖКУ), сведения о платеже для регистрации в ГИС ЖКХ.  Необходимо передавать при %[оплате ЖКУ](/developers/payment-acceptance/scenario-extensions/utility-payments). """  # noqa: E501

    __type = None
    """Код вида платежного поручения."""  # noqa: E501

    @property
    def type(self):
        """Возвращает type модели PaymentOrder.

        :return: type модели PaymentOrder.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """Устанавливает type модели PaymentOrder.

        :param value: type модели PaymentOrder.
        :type value: str
        """
        self.__type = value





# -*- coding: utf-8 -*-
from yookassa.domain.models.payment_data.statement import StatementType, Statement
from yookassa.domain.models.payment_data.statement.delivery_method.delivery_method import DeliveryMethod
from yookassa.domain.models.payment_data.statement.delivery_method.delivery_method_factory import DeliveryMethodFactory


class StatementPaymentOverview(Statement):
    """Квитанция по платежу. """  # noqa: E501

    __delivery_method = None
    """Данные о выбранном способе доставки справки. """  # noqa: E501

    def __init__(self, *args, **kwargs):
        super(StatementPaymentOverview, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not StatementType.PAYMENT_OVERVIEW:
            self.type = StatementType.PAYMENT_OVERVIEW

    @property
    def delivery_method(self):
        """Возвращает delivery_method модели StatementPaymentOverview.

        :return: delivery_method модели StatementPaymentOverview.
        :rtype: DeliveryMethod
        """
        return self.__delivery_method

    @delivery_method.setter
    def delivery_method(self, value):
        """Устанавливает delivery_method модели StatementPaymentOverview.

        :param value: delivery_method модели StatementPaymentOverview.
        :type value: DeliveryMethod
        """
        if isinstance(value, dict):
            self.__delivery_method = DeliveryMethodFactory().create(value, self.context())
        elif isinstance(value, DeliveryMethod):
            self.__delivery_method = value
        else:
            raise TypeError('Invalid delivery_method data type in StatementPaymentOverview.delivery_method')



# -*- coding: utf-8 -*-


class ErrorCode:
    """
    Константы, представляющие коды ошибок. Возможные значения:

    * yookassa.domain.common.ErrorCode.ERROR
    * yookassa.domain.common.ErrorCode.INVALID_REQUEST
    * yookassa.domain.common.ErrorCode.INVALID_CREDENTIALS
    * yookassa.domain.common.ErrorCode.FORBIDDEN
    * yookassa.domain.common.ErrorCode.NOT_FOUND
    * yookassa.domain.common.ErrorCode.GONE
    * yookassa.domain.common.ErrorCode.TOO_MANY_REQUESTS
    * yookassa.domain.common.ErrorCode.INTERNAL_SERVER_ERROR
    * yookassa.domain.common.ErrorCode.UNKNOWN
    """  # noqa: E501

    """
    Список допустимых значений
    """
    ERROR = 'error'
    """Общая ошибка"""

    INVALID_REQUEST = 'invalid_request'
    """Запрос не может быть обработан. Причиной может быть неправильный синтаксис запроса, ошибка в обязательных параметрах запроса, их отсутствие или неподдерживаемый метод."""  # noqa: E501

    INVALID_CREDENTIALS = 'invalid_credentials'
    """В заголовке Authorization указан неверный ключ."""

    FORBIDDEN = 'forbidden'
    """Секретный ключ указан верно, но не хватает прав для совершения операции."""

    NOT_FOUND = 'not_found'
    """Сущность не найдена."""

    GONE = 'gone'
    """Сущность была раньше, но была умышленно удалена и теперь недоступна."""

    TOO_MANY_REQUESTS = 'too_many_requests'
    """Слишком много запросов одновременно отправляется в API. Повторите запрос позже."""

    INTERNAL_SERVER_ERROR = 'internal_server_error'
    """Внутренняя ошибка сервера ЮKassa."""

    UNKNOWN = 'unknown'
    """Для неописанных кодов ошибок."""



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject


class AuthorizationDetails(BaseObject):
    """Данные об авторизации платежа при оплате банковской картой. Присутствуют только для этих способов оплаты: банковская карта, Mir Pay, SberPay, T-Pay. """  # noqa: E501

    __rrn = None
    """Retrieval Reference Number — уникальный идентификатор транзакции в системе эмитента."""  # noqa: E501

    __auth_code = None
    """Код авторизации. Выдается эмитентом и подтверждает проведение авторизации."""  # noqa: E501

    __three_d_secure = None
    """Данные о прохождении пользователем аутентификации по 3‑D Secure."""  # noqa: E501

    @property
    def rrn(self):
        """
        Возвращает rrn модели AuthorizationDetails.

        :return: rrn модели AuthorizationDetails.
        :rtype: str
        """
        return self.__rrn

    @rrn.setter
    def rrn(self, value):
        """
        Устанавливает rrn модели AuthorizationDetails.

        :param value: rrn модели AuthorizationDetails.
        :type value: str
        """
        self.__rrn = str(value)

    @property
    def auth_code(self):
        """
        Возвращает auth_code модели AuthorizationDetails.

        :return: auth_code модели AuthorizationDetails.
        :rtype: str
        """
        return self.__auth_code

    @auth_code.setter
    def auth_code(self, value):
        """
        Устанавливает auth_code модели AuthorizationDetails.

        :param value: auth_code модели AuthorizationDetails.
        :type value: str
        """
        self.__auth_code = str(value)

    @property
    def three_d_secure(self):
        """
        Возвращает three_d_secure модели AuthorizationDetails.

        :return: three_d_secure модели AuthorizationDetails.
        :rtype: ThreeDSecure
        """
        return self.__three_d_secure

    @three_d_secure.setter
    def three_d_secure(self, value):
        """
        Устанавливает three_d_secure модели AuthorizationDetails.

        :param value: three_d_secure модели AuthorizationDetails.
        :type value: ThreeDSecure
        """
        if isinstance(value, dict):
            self.__three_d_secure = ThreeDSecure(value)
        elif isinstance(value, ThreeDSecure):
            self.__three_d_secure = value
        else:
            raise TypeError('Invalid three_d_secure value type')


class ThreeDSecure(BaseObject):
    """
    Данные о прохождении пользователем аутентификации по 3‑D Secure для подтверждения платежа.
    """  # noqa: E501

    __applied = None
    """Отображение пользователю формы для прохождения аутентификации по 3‑D Secure. Возможные значения:  * ~`true` — ЮKassa отобразила пользователю форму, чтобы он мог пройти аутентификацию по 3‑D Secure; * ~`false` — платеж проходил без аутентификации по 3‑D Secure. """  # noqa: E501

    @property
    def applied(self):
        """
        Возвращает applied модели ThreeDSecure.

        :return: applied модели ThreeDSecure.
        :rtype: bool
        """
        return self.__applied

    @applied.setter
    def applied(self, value):
        """
        Устанавливает applied модели ThreeDSecure.

        :param value: applied модели ThreeDSecure.
        :type value: bool
        """
        self.__applied = bool(value)



# coding: utf-8
import re  # noqa: F401

from yookassa.domain.common import BaseObject


class MarkCodeInfo(BaseObject):
    """
    Код товара (тег в 54 ФЗ — 1163). Обязательный параметр, если одновременно выполняются эти условия:
      * вы используете Чеки от ЮKassa или онлайн-кассу, обновленную до ФФД 1.2;
      * товар нужно [маркировать](http://docs.cntd.ru/document/902192509).
    Должно быть заполнено хотя бы одно поле.
    """  # noqa: E501

    __mark_code_raw = None
    """
    Код товара в том виде, в котором он был прочитан сканером (тег в 54 ФЗ — 2000).
    Нужно передавать, если используете онлайн-кассу Orange Data.
    Пример: ~`010460406000590021N4N57RTCBUZTQ\\u001d2403054002410161218\\u001d1424010191ffd0\\u001g92tIAF/YVpU4roQS3M/m4z78yFq0nc/WsSmLeX6QkF/YVWwy5IMYAeiQ91Xa2m/fFSJcOkb2N+uUUtfr4n0mOX0Q==`
    """  # noqa: E501

    __unknown = None
    """Нераспознанный код товара (тег в 54 ФЗ — 1300)."""  # noqa: E501

    __ean_8 = None
    """Код товара в формате EAN-8 (тег в 54 ФЗ — 1301)."""  # noqa: E501

    __ean_13 = None
    """Код товара в формате EAN-13 (тег в 54 ФЗ — 1302)."""  # noqa: E501

    __itf_14 = None
    """Код товара в формате ITF-14 (тег в 54 ФЗ — 1303)."""  # noqa: E501

    __gs_10 = None
    """
    Код товара в формате GS1.0 (тег в 54 ФЗ — 1304).
    Можно передавать, если используете онлайн-кассу Orange Data, aQsi, Кит Инвест.
    """  # noqa: E501

    __gs_1m = None
    """Код товара в формате GS1.M (тег в 54 ФЗ — 1305)."""  # noqa: E501

    __short = None
    """Код товара в формате короткого кода маркировки (тег в 54 ФЗ — 1306)."""  # noqa: E501

    __fur = None
    """Контрольно-идентификационный знак мехового изделия (тег в 54 ФЗ — 1307)."""  # noqa: E501

    __egais_20 = None
    """Код товара в формате ЕГАИС-2.0 (тег в 54 ФЗ — 1308)."""  # noqa: E501

    __egais_30 = None
    """Код товара в формате ЕГАИС-3.0 (тег в 54 ФЗ — 1309)."""  # noqa: E501

    @property
    def mark_code_raw(self):
        """
        Возвращает mark_code_raw модели MarkCodeInfo.

        :return: mark_code_raw модели MarkCodeInfo.
        :rtype: str
        """
        return self.__mark_code_raw

    @mark_code_raw.setter
    def mark_code_raw(self, value):
        """
        Устанавливает mark_code_raw модели MarkCodeInfo.

        :param value: mark_code_raw модели MarkCodeInfo.
        :type value: str
        """
        self.__mark_code_raw = value

    @property
    def unknown(self):
        """
        Возвращает unknown модели MarkCodeInfo.

        :return: unknown модели MarkCodeInfo.
        :rtype: str
        """
        return self.__unknown

    @unknown.setter
    def unknown(self, value):
        """
        Устанавливает unknown модели MarkCodeInfo.

        :param value: unknown модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 32:
            raise ValueError("Invalid value for `unknown`, length must be less than or equal to `32`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `unknown`, length must be greater than or equal to `1`")  # noqa: E501
        self.__unknown = value

    @property
    def ean_8(self):
        """
        Возвращает ean_8 модели MarkCodeInfo.

        :return: ean_8 модели MarkCodeInfo.
        :rtype: str
        """
        return self.__ean_8

    @ean_8.setter
    def ean_8(self, value):
        """
        Устанавливает ean_8 модели MarkCodeInfo.

        :param value: ean_8 модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 8:
            raise ValueError("Invalid value for `ean_8`, length must be less than or equal to `8`")  # noqa: E501
        if value is not None and len(value) < 8:
            raise ValueError("Invalid value for `ean_8`, length must be greater than or equal to `8`")  # noqa: E501
        self.__ean_8 = value

    @property
    def ean_13(self):
        """
        Возвращает ean_13 модели MarkCodeInfo.

        :return: ean_13 модели MarkCodeInfo.
        :rtype: str
        """
        return self.__ean_13

    @ean_13.setter
    def ean_13(self, value):
        """
        Устанавливает ean_13 модели MarkCodeInfo.

        :param value: ean_13 модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 13:
            raise ValueError("Invalid value for `ean_13`, length must be less than or equal to `13`")  # noqa: E501
        if value is not None and len(value) < 13:
            raise ValueError("Invalid value for `ean_13`, length must be greater than or equal to `13`")  # noqa: E501
        self.__ean_13 = value

    @property
    def itf_14(self):
        """
        Возвращает itf_14 модели MarkCodeInfo.

        :return: itf_14 модели MarkCodeInfo.
        :rtype: str
        """
        return self.__itf_14

    @itf_14.setter
    def itf_14(self, value):
        """
        Устанавливает itf_14 модели MarkCodeInfo.

        :param value: itf_14 модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 14:
            raise ValueError("Invalid value for `itf_14`, length must be less than or equal to `14`")  # noqa: E501
        if value is not None and len(value) < 14:
            raise ValueError("Invalid value for `itf_14`, length must be greater than or equal to `14`")  # noqa: E501
        self.__itf_14 = value

    @property
    def gs_10(self):
        """
        Возвращает gs_10 модели MarkCodeInfo.

        :return: gs_10 модели MarkCodeInfo.
        :rtype: str
        """
        return self.__gs_10

    @gs_10.setter
    def gs_10(self, value):
        """
        Устанавливает gs_10 модели MarkCodeInfo.

        :param value: gs_10 модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 38:
            raise ValueError("Invalid value for `gs_10`, length must be less than or equal to `38`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `gs_10`, length must be greater than or equal to `1`")  # noqa: E501
        self.__gs_10 = value

    @property
    def gs_1m(self):
        """
        Возвращает gs_1m модели MarkCodeInfo.

        :return: gs_1m модели MarkCodeInfo.
        :rtype: str
        """
        return self.__gs_1m

    @gs_1m.setter
    def gs_1m(self, value):
        """
        Устанавливает gs_1m модели MarkCodeInfo.

        :param value: gs_1m модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 200:
            raise ValueError("Invalid value for `gs_1m`, length must be less than or equal to `200`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `gs_1m`, length must be greater than or equal to `1`")  # noqa: E501
        self.__gs_1m = value

    @property
    def short(self):
        """
        Возвращает short модели MarkCodeInfo.

        :return: short модели MarkCodeInfo.
        :rtype: str
        """
        return self.__short

    @short.setter
    def short(self, value):
        """
        Устанавливает short модели MarkCodeInfo.

        :param value: short модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 38:
            raise ValueError("Invalid value for `short`, length must be less than or equal to `38`")  # noqa: E501
        if value is not None and len(value) < 1:
            raise ValueError("Invalid value for `short`, length must be greater than or equal to `1`")  # noqa: E501
        self.__short = value

    @property
    def fur(self):
        """
        Возвращает fur модели MarkCodeInfo.

        :return: fur модели MarkCodeInfo.
        :rtype: str
        """
        return self.__fur

    @fur.setter
    def fur(self, value):
        """
        Устанавливает fur модели MarkCodeInfo.

        :param value: fur модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 20:
            raise ValueError("Invalid value for `fur`, length must be less than or equal to `20`")  # noqa: E501
        if value is not None and len(value) < 20:
            raise ValueError("Invalid value for `fur`, length must be greater than or equal to `20`")  # noqa: E501
        self.__fur = value

    @property
    def egais_20(self):
        """
        Возвращает egais_20 модели MarkCodeInfo.

        :return: egais_20 модели MarkCodeInfo.
        :rtype: str
        """
        return self.__egais_20

    @egais_20.setter
    def egais_20(self, value):
        """
        Устанавливает egais_20 модели MarkCodeInfo.

        :param value: egais_20 модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 33:
            raise ValueError("Invalid value for `egais_20`, length must be less than or equal to `33`")  # noqa: E501
        if value is not None and len(value) < 33:
            raise ValueError("Invalid value for `egais_20`, length must be greater than or equal to `33`")  # noqa: E501
        self.__egais_20 = value

    @property
    def egais_30(self):
        """
        Возвращает egais_30 модели MarkCodeInfo.

        :return: egais_30 модели MarkCodeInfo.
        :rtype: str
        """
        return self.__egais_30

    @egais_30.setter
    def egais_30(self, value):
        """
        Устанавливает egais_30 модели MarkCodeInfo.

        :param value: egais_30 модели MarkCodeInfo.
        :type value: str
        """
        if value is not None and len(value) > 14:
            raise ValueError("Invalid value for `egais_30`, length must be less than or equal to `14`")  # noqa: E501
        if value is not None and len(value) < 14:
            raise ValueError("Invalid value for `egais_30`, length must be greater than or equal to `14`")  # noqa: E501
        self.__egais_30 = value



# -*- coding: utf-8 -*-
import uuid

from yookassa.client import ApiClient
from yookassa.domain.common.http_verb import HttpVerb
from yookassa.domain.request.webhook_request import WebhookRequest
from yookassa.domain.response.webhook_response import WebhookResponse, WebhookList


class Webhook:
    """
    Класс, представляющий модель Webhook.
    """  # noqa: E501

    base_path = '/webhooks'

    def __init__(self):
        self.client = ApiClient()

    @classmethod
    def list(cls):
        """
        Возвращает список вебхуков

        :return: WebhookList Объект ответа, возвращаемого API при запросе списка вебхуков
        """
        instance = cls()
        path = cls.base_path

        response = instance.client.request(HttpVerb.GET, path)
        return WebhookList(response)

    @classmethod
    def add(cls, params, idempotency_key=None):
        """
        Добавление вебхука

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: WebhookResponse Объект ответа, возвращаемого API при запросе информации о вебхуке
        """
        instance = cls()
        path = cls.base_path
        if not idempotency_key:
            idempotency_key = uuid.uuid4()
        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = WebhookRequest(params)
        elif isinstance(params, WebhookRequest):
            params_object = params
        else:
            raise TypeError('Invalid params value type')

        response = instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return WebhookResponse(response)

    @classmethod
    def remove(cls, webhook_id, idempotency_key=None):
        """
        Удаление вебхука

        :param webhook_id: Уникальный идентификатор вебхука
        :param idempotency_key: Ключ идемпотентности
        :return: WebhookResponse Объект ответа, возвращаемого API при запросе информации о вебхуке
        """
        instance = cls()
        path = cls.base_path + '/' + webhook_id
        if not idempotency_key:
            idempotency_key = uuid.uuid4()
        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        response = instance.client.request(HttpVerb.DELETE, path, None, headers)
        return WebhookResponse(response)



# -*- coding: utf-8 -*-
"""Top-level package for YooKassa API Python Client Library."""

from yookassa.domain.response.payment_list_response import PaymentListResponse
from yookassa.domain.response.payment_response import PaymentResponse
from yookassa.domain.response.receipt_item_response import ReceiptItemResponse
from yookassa.domain.response.receipt_list_response import ReceiptListResponse
from yookassa.domain.response.receipt_response import ReceiptResponse
from yookassa.domain.response.refund_list_response import RefundListResponse
from yookassa.domain.response.refund_response import RefundResponse
from yookassa.domain.response.transfer_response import TransferResponse, TransferStatus
from yookassa.domain.response.webhook_response import WebhookResponse, WebhookList
from yookassa.domain.response.deal_response import DealResponse
from yookassa.domain.response.deal_list_response import DealListResponse
from yookassa.domain.response.invoice_response import InvoiceResponse
from yookassa.domain.response.payout_response import PayoutResponse
from yookassa.domain.response.personal_data_response import PersonalDataResponse
from yookassa.domain.response.sbp_bank_list_response import SbpBankListResponse
from yookassa.domain.response.self_employed_response import SelfEmployedResponse
from yookassa.domain.response.payment_method_response import PaymentMethodResponse



# -*- coding: utf-8 -*-
import requests
from base64 import b64encode
from requests.adapters import HTTPAdapter
from urllib3 import Retry

from yookassa import Configuration
from yookassa.domain.common import RequestObject, UserAgent
from yookassa.domain.exceptions import ApiError, BadRequestError, ForbiddenError, NotFoundError, \
    ResponseProcessingError, TooManyRequestsError, UnauthorizedError, InternalServerError, \
    GoneError


class ApiClient:
    """
    Класс клиента API.
    """  # noqa: E501

    def __init__(self):
        self.configuration = Configuration.instantiate()
        self.endpoint = Configuration.api_endpoint()
        self.shop_id = self.configuration.account_id
        self.shop_password = self.configuration.secret_key
        self.auth_token = self.configuration.auth_token
        self.timeout = self.configuration.timeout
        self.max_attempts = self.configuration.max_attempts

        self.user_agent = UserAgent()
        if self.configuration.agent_framework:
            self.user_agent.framework = self.configuration.agent_framework
        if self.configuration.agent_cms:
            self.user_agent.cms = self.configuration.agent_cms
        if self.configuration.agent_module:
            self.user_agent.module = self.configuration.agent_module

    def request(self, method="", path="", query_params=None, headers=None, body=None):
        """
        Подготовка запроса.

        :param method: HTTP метод
        :param path: URL запроса
        :param query_params: Массив GET параметров запроса
        :param headers: Массив заголовков запроса
        :param body: Тело запроса
        :rtype: str
        """
        if isinstance(body, RequestObject):
            body.validate()
            body = dict(body)

        request_headers = self.prepare_request_headers(headers)

        try:
            raw_response = self.execute(body, method, path, query_params, request_headers)
        except Exception as e:
            self.__handle_error(e.response)

        if raw_response.status_code != 200:
            self.__handle_error(raw_response)

        return raw_response.json()

    def execute(self, body, method, path, query_params, request_headers):
        """
        Выполнение запроса.

        :param body: Тело запроса
        :param method: HTTP метод
        :param path: URL запроса
        :param query_params: Массив GET параметров запроса
        :param request_headers: Массив заголовков запроса
        :return: Response
        """
        session = self.get_session()
        self.log_request(body, method, path, query_params, request_headers)

        raw_response = session.request(
            method,
            self.endpoint + path,
            params=query_params,
            headers=request_headers,
            json=body,
            verify=self.configuration.verify
        )

        session.close()
        self.log_response(raw_response.content, self.get_response_info(raw_response), raw_response.headers)

        return raw_response

    def get_session(self):
        """
        Получение сессии.

        :return: Session
        """
        session = requests.Session()
        retries = Retry(total=self.max_attempts,
                        backoff_factor=self.timeout / 1000,
                        allowed_methods=['POST'],
                        status_forcelist=[202])
        session.mount('https://', HTTPAdapter(max_retries=retries))
        return session

    def prepare_request_headers(self, headers):
        """
        Подготовка заголовков запроса.

        :param headers: Массив заголовков запроса
        :return: dict[str, str]
        """
        request_headers = {'Content-type': 'application/json'}
        if self.auth_token is not None:
            auth_headers = {"Authorization": "Bearer " + self.auth_token}
        else:
            auth_headers = {"Authorization": self.basic_auth(self.shop_id, self.shop_password)}

        request_headers.update(auth_headers)

        request_headers.update({"YM-User-Agent": self.user_agent.get_header_string()})

        if isinstance(headers, dict):
            request_headers.update(headers)
        return request_headers

    @staticmethod
    def basic_auth(username, password):
        """
        Формирование токена для Basic авторизации.

        :param username: ShopId магазина
        :param password: Секретный ключ магазина
        :return: str
        """
        token = b64encode(f"{username}:{password}".encode('utf-8')).decode("ascii")
        return f'Basic {token}'

    @staticmethod
    def __handle_error(raw_response):
        """
        Выбрасывает исключение по коду ошибки.
        """
        http_code = raw_response.status_code
        if http_code == BadRequestError.HTTP_CODE:
            raise BadRequestError(raw_response.json())
        elif http_code == ForbiddenError.HTTP_CODE:
            raise ForbiddenError(raw_response.json())
        elif http_code == NotFoundError.HTTP_CODE:
            raise NotFoundError(raw_response.json())
        elif http_code == TooManyRequestsError.HTTP_CODE:
            raise TooManyRequestsError(raw_response.json())
        elif http_code == UnauthorizedError.HTTP_CODE:
            raise UnauthorizedError(raw_response.json())
        elif http_code == ResponseProcessingError.HTTP_CODE:
            raise ResponseProcessingError(raw_response.json())
        elif http_code == InternalServerError.HTTP_CODE:
            raise InternalServerError(raw_response.json())
        elif http_code == GoneError.HTTP_CODE:
            raise GoneError(raw_response.json())
        else:
            raise ApiError(raw_response.json())

    @staticmethod
    def get_response_info(response):
        """
        Получение информации из ответа.

        :param response: Объект ответа
        :return: dict[str, Any]
        """
        return {
            "apparent_encoding": response.apparent_encoding,
            "cookies": response.cookies,
            "elapsed": response.elapsed,
            "encoding": response.encoding,
            "is_permanent_redirect": response.is_permanent_redirect,
            "is_redirect": response.is_redirect,
            "ok": response.ok,
            "raise_for_status": response.raise_for_status(),
            "reason": response.reason,
            "status_code": response.status_code,
            "url": response.url,
        }

    def log_request(self, body, method, path, query_params, headers):
        """
        Логирование запроса.

        :param body: Тело запроса
        :param method: HTTP метод
        :param path: URL запроса
        :param query_params: Массив GET параметров запроса
        :param headers: Массив заголовков запроса
        """
        if Configuration.logger:
            context = {}
            if query_params:
                context['_params'] = query_params
            if body:
                context['_body'] = body
            if headers:
                context['_headers'] = headers

            message = 'Send request: {} {} '.format(str(method), self.endpoint + path)
            Configuration.logger.info(message, context=context)

    @staticmethod
    def log_response(body, info, headers):
        """
        Логирование ответа.

        :param body: Тело ответа
        :param info: Массив с данными ответа
        :param headers: Массив заголовков ответа
        """
        if Configuration.logger:
            context = {}
            if body:
                context['_body'] = body
            if headers:
                context['_headers'] = headers

            message = 'Response with code [{}] received.'.format(info['status_code'])
            Configuration.logger.info(message, context=context)



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.models import Settlement


class PaymentDealInfo(BaseObject):
    """
    Сделка, в рамках которой нужно провести платеж.
    """  # noqa: E501

    __id = None
    """Идентификатор сделки."""  # noqa: E501

    __settlements = []
    """Данные о распределении денег."""  # noqa: E501

    @property
    def id(self):
        """
        Возвращает id модели PaymentDealInfo.

        :return: id модели PaymentDealInfo.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """
        Устанавливает id модели PaymentDealInfo.

        :param value: id модели PaymentDealInfo.
        :type value: str
        """
        self.__id = value

    @property
    def settlements(self):
        """
        Возвращает settlements модели PaymentDealInfo.

        :return: settlements модели PaymentDealInfo.
        :rtype: list[Settlement]
        """
        return self.__settlements

    @settlements.setter
    def settlements(self, value):
        """
        Устанавливает settlements модели PaymentDealInfo.

        :param value: settlements модели PaymentDealInfo.
        :type value: list[Settlement]
        """
        if isinstance(value, list):
            settlements = []
            for item in value:
                if isinstance(item, dict):
                    settlements.append(Settlement(item))
                elif isinstance(item, Settlement):
                    settlements.append(item)
                else:
                    raise TypeError('Invalid item type in deal.settlements')

            self.__settlements = settlements
        elif value is None:
            self.__settlements = []
        else:
            raise TypeError('Invalid settlements value type in PaymentDealInfo')


class PayoutDealInfo(BaseObject):
    """
    Сделка, в рамках которой нужно провести выплату. Необходимо передавать, если вы проводите %[Безопасную сделку](/developers/solutions-for-platforms/safe-deal/basics)
    """  # noqa: E501

    __id = None
    """Идентификатор сделки."""  # noqa: E501

    @property
    def id(self):
        """
        Возвращает id модели PayoutDealInfo.

        :return: id модели PayoutDealInfo.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """
        Устанавливает id модели PayoutDealInfo.

        :param value: id модели PayoutDealInfo.
        :type value: str
        """
        self.__id = value


class RefundDealInfo(BaseObject):
    """
    Данные о сделке, в составе которой проходит возврат. Присутствует, если вы проводите %[Безопасную сделку](/developers/solutions-for-platforms/safe-deal/basics).
    """  # noqa: E501

    __id = None
    """Идентификатор сделки."""  # noqa: E501

    __refund_settlements = []
    """Данные о распределении денег."""  # noqa: E501

    @property
    def id(self):
        """
        Возвращает id модели RefundDealInfo.

        :return: id модели RefundDealInfo.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """
        Устанавливает id модели RefundDealInfo.

        :param value: id модели RefundDealInfo.
        :type value: str
        """
        self.__id = value

    @property
    def refund_settlements(self):
        """
        Возвращает refund_settlements модели RefundDealInfo.

        :return: refund_settlements модели RefundDealInfo.
        :rtype: list[Settlement]
        """
        return self.__refund_settlements

    @refund_settlements.setter
    def refund_settlements(self, value):
        """
        Устанавливает refund_settlements модели RefundDealInfo.

        :param value: refund_settlements модели RefundDealInfo.
        :type value: list[Settlement]
        """
        if isinstance(value, list):
            settlements = []
            for item in value:
                if isinstance(item, dict):
                    settlements.append(Settlement(item))
                elif isinstance(item, Settlement):
                    settlements.append(item)
                else:
                    raise TypeError('Invalid item type in deal.refund_settlements')

            self.__refund_settlements = settlements
        elif value is None:
            self.__refund_settlements = []
        else:
            raise TypeError('Invalid refund_settlements value type in RefundDealInfo')


class RefundDealData(BaseObject):
    """
    Данные о сделке, в составе которой проходит возврат. Необходимо передавать, если вы проводите %[Безопасную сделку](/developers/solutions-for-platforms/safe-deal/basics).
    """  # noqa: E501

    __refund_settlements = []
    """Данные о распределении денег."""  # noqa: E501

    @property
    def refund_settlements(self):
        """
        Возвращает refund_settlements модели RefundDealData.

        :return: refund_settlements модели RefundDealData.
        :rtype: list[Settlement]
        """
        return self.__refund_settlements

    @refund_settlements.setter
    def refund_settlements(self, value):
        """
        Устанавливает refund_settlements модели RefundDealData.

        :param value: refund_settlements модели RefundDealData.
        :type value: list[Settlement]
        """
        if isinstance(value, list):
            settlements = []
            for item in value:
                if isinstance(item, dict):
                    settlements.append(Settlement(item))
                elif isinstance(item, Settlement):
                    settlements.append(item)
                else:
                    raise TypeError('Invalid item type in deal.refund_settlements')

            self.__refund_settlements = settlements
        elif value is None:
            self.__refund_settlements = []
        else:
            raise TypeError('Invalid refund_settlements value type in RefundDealData')


class DealType:
    """
    Тип сделки.
    """  # noqa: E501

    """
    Список допустимых значений
    """
    SAFE_DEAL = 'safe_deal'
    """Безопасная сделка."""


class DealStatus:
    """
    Статус сделки.
    """  # noqa: E501

    """
    Список допустимых значений
    """
    OPENED = 'opened'
    """Сделка открыта. Можно выполнять платежи, возвраты и выплаты в составе сделки."""
    CLOSED = 'closed'
    """Сделка закрыта. Вознаграждение перечислено продавцу и платформе или оплата возвращена покупателю. Нельзя выполнять платежи, возвраты и выплаты в составе сделки."""


class FeeMoment:
    """
    Момент перечисления вам вознаграждения платформы.
    """  # noqa: E501

    """
    Список допустимых значений
    """
    PAYMENT_SUCCEEDED = 'payment_succeeded'
    """Вознаграждение после успешной оплаты."""
    DEAL_CLOSED = 'deal_closed'
    """Вознаграждение при закрытии сделки после успешной выплаты."""



# coding: utf-8

from yookassa.domain.common import BaseObject
from yookassa.domain.models import Amount


class InvoiceStatus:
    """
    Статус счета.
    Возможные значения:
    - `pending` — счет создан и ожидает успешной оплаты;
    - `succeeded` — счет успешно оплачен, есть связанный платеж в статусе ~`succeeded` (финальный и неизменяемый статус для платежей в одну стадию);
    - `canceled` — вы отменили счет, успешный платеж по нему не поступил или был отменен (при оплате в две стадии) либо истек срок действия счета (финальный и неизменяемый статус).

    Подробнее про [жизненный цикл счета](/developers/payment-acceptance/scenario-extensions/invoices/basics#invoice-status)
    """  # noqa: E501

    """
    Список допустимых значений
    """
    PENDING = "pending"
    """Счет создан и ожидает успешной оплаты"""
    SUCCEEDED = "succeeded"
    """Счет успешно оплачен, есть связанный платеж в статусе ~`succeeded`"""
    CANCELED = "canceled"
    """Вы отменили счет, успешный платеж по нему не поступил или был отменен"""


class DeliveryMethodType:
    """
    Класс, представляющий модель DeliveryMethodType.

    Код способа доставки счета пользователю.
    Возможные значения:
    - `self` — Самостоятельно.
    Подробнее про [жизненный цикл счета](https://yookassa.ru/developers/payment-acceptance/scenario-extensions/invoices/basics)
    """  # noqa: E501

    """
    Список допустимых значений
    """
    SELF = "self"
    """Самостоятельно"""
    UNKNOWN = "unknown"
    """Для неизвестных методов доставки счета пользователю"""


class LineItem(BaseObject):
    """Данные о товаре или услуге в корзине."""  # noqa: E501

    __description = None
    """Название товара или услуги (от 1 до 128 символов). Пользователь увидит его на странице счета перед оплатой."""  # noqa: E501

    __price = None
    """Полная цена товара или услуги. Пользователь увидит ее на странице счета перед оплатой."""  # noqa: E501

    __discount_price = None
    """Итоговая цена товара с учетом скидки. Если передана, то на странице счета цена отобразится с учетом скидки. Не нужно передавать, если пользователь оплачивает полную стоимость товара или услуги."""  # noqa: E501

    __quantity = None
    """Количество товара. Можно передать целое или дробное число. Разделитель дробной части — точка, разделитель тысяч отсутствует, максимум три знака после точки. Пример: ~`5.000` """  # noqa: E501

    @property
    def description(self):
        """Возвращает description модели LineItem.

        :return: description модели LineItem.
        :rtype: str
        """
        return self.__description

    @description.setter
    def description(self, value):
        """Устанавливает description модели LineItem.

        :param value: description модели LineItem.
        :type value: str
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `description`, must not be `None`")  # noqa: E501
        self.__description = value

    @property
    def price(self):
        """Возвращает price модели LineItem.

        :return: price модели LineItem.
        :rtype: LineItemPrice
        """
        return self.__price

    @price.setter
    def price(self, value):
        """Устанавливает price модели LineItem.

        :param value: price модели LineItem.
        :type value: LineItemPrice
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `price`, must not be `None`")  # noqa: E501
        if isinstance(value, dict):
            self.__price = Amount(value)
        elif isinstance(value, Amount):
            self.__price = value
        else:
            raise TypeError('Invalid price data type in LineItem.price')

    @property
    def discount_price(self):
        """Возвращает discount_price модели LineItem.

        :return: discount_price модели LineItem.
        :rtype: LineItemDiscountPrice
        """
        return self.__discount_price

    @discount_price.setter
    def discount_price(self, value):
        """Устанавливает discount_price модели LineItem.

        :param value: discount_price модели LineItem.
        :type value: LineItemDiscountPrice
        """
        if isinstance(value, dict):
            self.__discount_price = Amount(value)
        elif isinstance(value, Amount):
            self.__discount_price = value
        else:
            raise TypeError('Invalid discount_price data type in LineItem.discount_price')

    @property
    def quantity(self):
        """Возвращает quantity модели LineItem.

        :return: quantity модели LineItem.
        :rtype: float
        """
        return self.__quantity

    @quantity.setter
    def quantity(self, value):
        """Устанавливает quantity модели LineItem.

        :param value: quantity модели LineItem.
        :type value: float
        """
        if value is None:  # noqa: E501
            raise ValueError("Invalid value for `quantity`, must not be `None`")  # noqa: E501
        self.__quantity = value


class PaymentDetails(BaseObject):
    """Данные о платеже по выставленному счету. Присутствуют, только если платеж успешно %[подтвержден пользователем](/developers/payment-acceptance/getting-started/payment-process#user-confirmation). """  # noqa: E501

    __id = None
    """Идентификатор платежа в ЮKassa."""  # noqa: E501

    __status = None
    """Статус платежа."""  # noqa: E501

    @property
    def id(self):
        """Возвращает id модели PaymentDetails.

        :return: id модели PaymentDetails.
        :rtype: str
        """
        return self.__id

    @id.setter
    def id(self, value):
        """Устанавливает id модели PaymentDetails.

        :param value: id модели PaymentDetails.
        :type value: str
        """
        self.__id = value

    @property
    def status(self):
        """Возвращает status модели PaymentDetails.

        :return: status модели PaymentDetails.
        :rtype: PaymentStatus
        """
        return self.__status

    @status.setter
    def status(self, value):
        """Устанавливает status модели PaymentDetails.

        :param value: status модели PaymentDetails.
        :type value: PaymentStatus
        """
        self.__status = value



# -*- coding: utf-8 -*-
import json
import logging
import sys
import time


class AppFormatter(logging.Formatter):
    """
    Класс AppFormatter.
    """  # noqa: E501

    def __init__(self, fmt=None, datefmt=None, style='%'):
        __tz = time.strftime('%z')
        if not fmt:
            fmt = "[%(asctime)s.%(msecs)03d" + __tz + "] %(levelname)s [ ] [%(name)s] %(message)s %(context)s"
        if not datefmt:
            datefmt = "%Y-%m-%dT%H:%M:%S"
        super(AppFormatter, self).__init__(fmt, datefmt, style)


class JsonFormatter(logging.Formatter):
    """
    Класс JsonFormatter.
    """  # noqa: E501

    def __init__(self, fmt=None, datefmt=None, style='%'):
        __tz = time.strftime('%z')
        if not fmt:
            fmt = json.dumps(dict({
                "message": "%(message)s", "level": "%(levelname)s", "channel": "%(name)s",
                "extra": [], "app_name": "app_main", "app_type": "python", "log_type": "code",
                "es_index_name": "python-main", "timestamp": "%(asctime)s.%(msecs)03d" + __tz,
                "context": "%(context)s"
            }))
        if not datefmt:
            datefmt = "%Y-%m-%dT%H:%M:%S"
        super(JsonFormatter, self).__init__(fmt, datefmt, style)


class StreamHandler(logging.StreamHandler):
    """
    Класс StreamHandler.
    """  # noqa: E501

    level = 0

    def __init__(self, stream=sys.stdout):
        super(StreamHandler, self).__init__(stream)
        self.setFormatter(JsonFormatter())


class AppHandler(logging.FileHandler):
    """
    Класс AppHandler.
    """  # noqa: E501

    level = 0

    def __init__(self, filename='app.log', mode='a', encoding='utf-8', delay=False):
        super(AppHandler, self).__init__(filename, mode, encoding, delay)
        self.setFormatter(JsonFormatter())


class AppLogger(logging.Logger):
    """
    Класс AppLogger.
    """  # noqa: E501

    def __init__(self, name, level='DEBUG'):
        super(AppLogger, self).__init__(name, level)
        self.addHandler(StreamHandler())
        self.addHandler(AppHandler())



# -*- coding: utf-8 -*-
from yookassa.domain.common.response_object import ResponseObject
from yookassa.domain.response.refund_response import RefundResponse


class RefundListResponse(ResponseObject):
    """
    Объект ответа, возвращаемого API при запросе списка возвратов.
    """  # noqa: E501

    __type = None
    """Формат выдачи результатов запроса. Возможное значение: ~`list` (список). """  # noqa: E501

    __items = None
    """Массив платежей."""  # noqa: E501

    __next_cursor = None
    """Указатель на следующий фрагмент списка. Обязательный параметр, если размер списка больше размера выдачи (`limit`) и конец выдачи не достигнут."""  # noqa: E501

    @property
    def type(self):
        """
        Возвращает type модели RefundListResponse.

        :return: type модели RefundListResponse.
        :rtype: str
        """
        return self.__type

    @type.setter
    def type(self, value):
        """
        Устанавливает type модели RefundListResponse.

        :param value: type модели RefundListResponse.
        :type value: str
        """
        self.__type = value

    @property
    def next_cursor(self):
        """
        Возвращает next_cursor модели RefundListResponse.

        :return: next_cursor модели RefundListResponse.
        :rtype: str
        """
        return self.__next_cursor

    @next_cursor.setter
    def next_cursor(self, value):
        """
        Устанавливает next_cursor модели RefundListResponse.

        :param value: next_cursor модели RefundListResponse.
        :type value: str
        """
        self.__next_cursor = value

    @property
    def items(self):
        """
        Возвращает items модели RefundListResponse.

        :return: items модели RefundListResponse.
        :rtype: list[RefundResponse]
        """
        return self.__items

    @items.setter
    def items(self, value):
        """
        Устанавливает items модели RefundListResponse.

        :param value: items модели RefundListResponse.
        :type value: list[RefundResponse]
        """
        if isinstance(value, list):
            self.__items = [RefundResponse(refund) for refund in value]
        else:
            self.__items = value



# -*- coding: utf-8 -*-
from decimal import Decimal, ROUND_HALF_UP

from yookassa.domain.common import BaseObject


class Amount(BaseObject):
    """
    Сумма определенная в валюте.
    """  # noqa: E501

    __value = None
    """Сумма в выбранной валюте. Всегда дробное значение. Разделитель дробной части — точка, разделитель тысяч отсутствует. Количество знаков после точки зависит от выбранной валюты. Пример: ~`1000.00`. """  # noqa: E501

    __currency = None
    """Трехбуквенный код валюты в формате ISO-4217. Пример: RUB."""  # noqa: E501

    @property
    def value(self):
        """
        Возвращает value модели Amount.

        :return: value модели Amount.
        :rtype: Decimal
        """
        return self.__value

    @value.setter
    def value(self, value):
        """
        Устанавливает value модели Amount.

        :param value: value модели Amount.
        :type value: Decimal
        """
        self.__value = Decimal(str(float(value))).quantize(Decimal('1.11'), rounding=ROUND_HALF_UP)

    @property
    def currency(self):
        """
        Возвращает currency модели Amount.

        :return: currency модели Amount.
        :rtype: str
        """
        return self.__currency

    @currency.setter
    def currency(self, value):
        """
        Устанавливает currency модели Amount.

        :param value: currency модели Amount.
        :type value: str
        """
        self.__currency = str(value)



# -*- coding: utf-8 -*-
import uuid

from yookassa.client import ApiClient
from yookassa.domain.common import HttpVerb
from yookassa.domain.request import CapturePaymentRequest, PaymentRequest
from yookassa.domain.response import PaymentListResponse, PaymentResponse


class Payment:
    """
    Класс, представляющий модель Payment.
    """  # noqa: E501

    base_path = '/payments'

    CMS_NAME = 'yookassa_sdk_python'

    def __init__(self):
        self.client = ApiClient()

    @classmethod
    def find_one(cls, payment_id):
        """
        Возвращает информацию о платеже

        :param payment_id: Уникальный идентификатор платежа
        :return: PaymentResponse Объект ответа, возвращаемого API при запросе платежа
        """
        instance = cls()
        if not isinstance(payment_id, str) or not payment_id:
            raise ValueError('Invalid payment_id value')

        path = instance.base_path + '/' + payment_id
        response = instance.client.request(HttpVerb.GET, path)
        return PaymentResponse(response)

    @classmethod
    def create(cls, params, idempotency_key=None):
        """
        Создание платежа

        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: PaymentResponse Объект ответа, возвращаемого API при запросе платежа
        """
        instance = cls()
        path = cls.base_path

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = PaymentRequest(params)
        elif isinstance(params, PaymentRequest):
            params_object = params
        else:
            raise TypeError('Invalid params value type')

        params_object = instance.add_default_cms_name(params_object)

        response = instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return PaymentResponse(response)

    @classmethod
    def capture(cls, payment_id, params=None, idempotency_key=None):
        """
        Подтверждение платежа

        :param payment_id: Уникальный идентификатор платежа
        :param params: Данные передаваемые в API
        :param idempotency_key: Ключ идемпотентности
        :return: PaymentResponse Объект ответа, возвращаемого API при запросе платежа
        """
        instance = cls()
        if not isinstance(payment_id, str) or not payment_id:
            raise ValueError('Invalid payment_id value')

        path = instance.base_path + '/' + payment_id + '/capture'

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = CapturePaymentRequest(params)
        elif isinstance(params, CapturePaymentRequest):
            params_object = params
        else:
            params_object = None

        response = instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return PaymentResponse(response)

    @classmethod
    def cancel(cls, payment_id, idempotency_key=None):
        """
        Отмена платежа

        :param payment_id: Уникальный идентификатор платежа
        :param idempotency_key: Ключ идемпотентности
        :return: PaymentResponse Объект ответа, возвращаемого API при запросе платежа
        """
        instance = cls()
        if not isinstance(payment_id, str) or not payment_id:
            raise ValueError('Invalid payment_id value')

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        path = instance.base_path + '/' + payment_id + '/cancel'
        headers = {
            'Idempotence-Key': str(idempotency_key)
        }
        response = instance.client.request(HttpVerb.POST, path, None, headers)
        return PaymentResponse(response)

    @classmethod
    def list(cls, params=None):
        """
        Возвращает список платежей

        :param params: Данные передаваемые в API
        :return: PaymentListResponse Объект ответа, возвращаемого API при запросе списка платежей
        """
        if params is None:
            params = {}
        instance = cls()
        path = cls.base_path

        response = instance.client.request(HttpVerb.GET, path, params)
        return PaymentListResponse(response)

    def add_default_cms_name(cls, params_object):
        """
        Добавляет cms_name в metadata со значение по умолчанию

        :param params_object: Данные передаваемые в API
        :return: PaymentRequest Объект запроса к API
        """
        if not params_object.metadata:
            params_object.metadata = {'cms_name': cls.CMS_NAME}

        if 'cms_name' not in params_object.metadata:
            params_object.metadata['cms_name'] = cls.CMS_NAME

        return params_object
